"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LOGGER_CONFIG = exports.TASK_LOG_FILE = void 0;
var path = require("path");
var stackTrace = require("stack-trace");
exports.TASK_LOG_FILE = path.resolve(__dirname, '..', 'logs', 'Task.log');
var getLogPid = function () {
    var pid = process.pid.toString();
    return (pid + '     ').substring(0, 5);
};
var getLogLevel = function (loggingEvent) {
    var resultLength = 5;
    var level = loggingEvent.level.toString();
    return '[' + (level + Array(resultLength + 1).join(' ')).substring(0, resultLength) + ']';
};
var getLogContext = function (loggingEvent) {
    var resultLength = 30;
    var trace = stackTrace.get();
    if (trace.length <= 8)
        return '[Unknown context]';
    var caller = trace[8];
    var fileName = path.basename(caller.getFileName() || '');
    var lineNumber = caller.getLineNumber() || '';
    var context = (fileName + ':' + lineNumber);
    context = (context.length > resultLength ? '...' + context.slice(-1 * (resultLength - ('...').length)) : context);
    return '[' + (context + Array(resultLength + 1).join(' ')).substring(0, resultLength) + ']';
};
exports.LOGGER_CONFIG = {
    appenders: [
        {
            type: 'file',
            filename: path.resolve(__dirname, '..', 'logs', 'AccessLog.log'),
            maxLogSize: 20 * 1000 * 1024,
            backups: 5,
            category: 'AccessLog',
            layout: {
                type: 'pattern',
                pattern: '%x{level} %d{ISO8601} %x{pid} %m',
                tokens: {
                    pid: getLogPid,
                    level: getLogLevel,
                    context: getLogContext
                }
            }
        },
        {
            type: 'file',
            filename: path.resolve(__dirname, '..', 'logs', 'ServerScripts.log'),
            maxLogSize: 20 * 1000 * 1024,
            backups: 5,
            category: 'ServerScripts',
            layout: {
                type: 'pattern',
                pattern: '%x{level} %d{ISO8601} %x{pid} %x{context} %m',
                tokens: {
                    pid: getLogPid,
                    level: getLogLevel,
                    context: getLogContext
                }
            }
        },
        {
            type: 'file',
            filename: path.resolve(__dirname, '..', 'logs', 'PublicScripts.log'),
            maxLogSize: 20 * 1000 * 1024,
            backups: 15,
            category: 'PublicScripts',
            layout: {
                type: 'pattern',
                pattern: '%x{level} %d{ISO8601} %x{pid} %x{context} %m',
                tokens: {
                    pid: getLogPid,
                    level: getLogLevel,
                    context: getLogContext
                }
            }
        },
        {
            type: 'file',
            filename: path.resolve(__dirname, '..', 'logs', 'DoubleClick.log'),
            maxLogSize: 20 * 1000 * 1024,
            backups: 5,
            category: 'DoubleClick',
            layout: {
                type: 'pattern',
                pattern: '%x{level} %d{ISO8601} %x{pid} %x{context} %m',
                tokens: {
                    pid: getLogPid,
                    level: getLogLevel,
                    context: getLogContext
                }
            }
        },
        {
            type: 'file',
            filename: path.resolve(__dirname, '..', 'logs', 'Sql.log'),
            maxLogSize: 20 * 1000 * 1024,
            backups: 5,
            category: 'Sql',
            layout: {
                type: 'pattern',
                pattern: '%x{level} %d{ISO8601} %x{pid} %x{context} %m',
                tokens: {
                    pid: getLogPid,
                    level: getLogLevel,
                    context: getLogContext
                }
            }
        },
        {
            type: 'file',
            filename: path.resolve(__dirname, '..', 'logs', 'Redis.log'),
            maxLogSize: 20 * 1000 * 1024,
            backups: 5,
            category: 'Redis',
            layout: {
                type: 'pattern',
                pattern: '%x{level} %d{ISO8601} %x{pid} %x{context} %m',
                tokens: {
                    pid: getLogPid,
                    level: getLogLevel,
                    context: getLogContext
                }
            }
        },
        {
            type: 'file',
            filename: path.resolve(__dirname, '..', 'logs', 'Task.log'),
            maxLogSize: 20 * 1000 * 1024,
            backups: 5,
            category: 'Task',
            layout: {
                type: 'pattern',
                pattern: '%x{level} %d{ISO8601} %x{pid} %x{context} %m',
                tokens: {
                    pid: getLogPid,
                    level: getLogLevel,
                    context: getLogContext
                }
            }
        },
        {
            type: 'file',
            filename: path.resolve(__dirname, '..', 'logs', 'Cleaner.log'),
            maxLogSize: 20 * 1000 * 1024,
            backups: 5,
            category: 'Cleaner',
            layout: {
                type: 'pattern',
                pattern: '%x{level} %d{ISO8601} %x{pid} %x{context} %m',
                tokens: {
                    pid: getLogPid,
                    level: getLogLevel,
                    context: getLogContext
                }
            }
        },
        {
            type: 'file',
            filename: path.resolve(__dirname, '..', 'logs', 'IO.log'),
            maxLogSize: 20 * 1000 * 1024,
            backups: 5,
            category: 'IO',
            layout: {
                type: 'pattern',
                pattern: '%x{level} %d{ISO8601} %x{pid} %x{context} %m',
                tokens: {
                    pid: getLogPid,
                    level: getLogLevel,
                    context: getLogContext
                }
            }
        }
    ]
};
