"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BulkInteractionEvaluationPlan = exports.GetInteractionsEvaluationPlan = exports.DeleteEvaluationPlanInteractionByTypeAndId = exports.GetEvaluationPlanInteractionByTypeAndId = exports.InteractionEvaluationPlanMSSql = void 0;
var moment = require("moment");
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function InteractionEvaluationPlanMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.InteractionEvaluationPlanMSSql = InteractionEvaluationPlanMSSql;
function GetEvaluationPlanInteractionByTypeAndId(vcc, evaluationPlanId, interactionType, interactionId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .input('InteractionType', sql.VarChar(50), interactionType)
        .input('InteractionId', sql.VarChar(250), interactionId)
        .execute('GetEvaluationPlanInteractionByTypeAndId')
        .then(function (recordsets) {
        logger.debug('[EvaluationPlanMSSql::GetEvaluationPlanInteractionByTypeAndId] EvaluationPlan::VCC => ' + evaluationPlanId + '::' + vcc);
        var recordset = (recordsets === null || recordsets === void 0 ? void 0 : recordsets.length) ? recordsets[0] : null;
        var interactionEvaluationPlan = (recordset === null || recordset === void 0 ? void 0 : recordset.length) ? recordset[0] : null;
        return Promise.resolve(interactionEvaluationPlan);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Error executing: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[EvaluationPlanMSSql::GetEvaluationPlanByName] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetEvaluationPlanInteractionByTypeAndId = GetEvaluationPlanInteractionByTypeAndId;
function DeleteEvaluationPlanInteractionByTypeAndId(vcc, evaluationPlanId, interactionType, interactionId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('EvaluationPlanId', sql.VarChar(50), evaluationPlanId)
        .input('InteractionType', sql.VarChar(50), interactionType)
        .input('InteractionId', sql.VarChar(250), interactionId)
        .execute('DeleteEvaluationPlanInteractionByTypeAndId')
        .then(function (recordsets) {
        logger.debug('[EvaluationPlanMSSql::DeleteEvaluationPlanInteractionByTypeAndId] EvaluationPlan::VCC => ' + evaluationPlanId + '::' + vcc);
        var recordset = (recordsets === null || recordsets === void 0 ? void 0 : recordsets.length) ? recordsets[0] : null;
        var output = (recordset === null || recordset === void 0 ? void 0 : recordset.length) ? recordset[0] : null;
        return Promise.resolve((output === null || output === void 0 ? void 0 : output.result) === 1);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error on \'' + err.procName + '\': ' : 'Error executing: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[EvaluationPlanMSSql::DeleteEvaluationPlanInteractionByTypeAndId] ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteEvaluationPlanInteractionByTypeAndId = DeleteEvaluationPlanInteractionByTypeAndId;
function GetInteractionsEvaluationPlan(vcc, evaluationPlanId) {
    return new sql.Request(sqlConn)
        .input('Instance', sql.VarChar(50), vcc)
        .input('IdEvaluationPlan', sql.VarChar(50), evaluationPlanId)
        .execute('GetInteractionsEvaluationPlan')
        .then(function (recordsets) {
        var recordset = (recordsets === null || recordsets === void 0 ? void 0 : recordsets.length) ? recordsets[0] : null;
        return Promise.resolve(recordset);
    })
        .catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractionsEvaluationPlan = GetInteractionsEvaluationPlan;
function BulkInteractionEvaluationPlan(vcc, evaluationPlan) {
    var table = SetInteractionEvaluationPlanTemporaryTable(vcc, evaluationPlan);
    return BulkInteractionEvaluationPlanTransaction(vcc, table);
}
exports.BulkInteractionEvaluationPlan = BulkInteractionEvaluationPlan;
function SetInteractionEvaluationPlanTemporaryTable(vcc, evaluationPlan) {
    var table = new sql.Table('#InteractionEvaluationPlanTemp');
    table.create = true;
    table.columns.add('VCC', sql.VarChar(50), { nullable: true });
    table.columns.add('IdEvaluationPlan', sql.VarChar(50), { nullable: true });
    table.columns.add('IdAgent', sql.VarChar(sql.MAX), { nullable: true });
    table.columns.add('AgentName', sql.VarChar(sql.MAX), { nullable: true });
    table.columns.add('IdCampaign', sql.VarChar(sql.MAX), { nullable: true });
    table.columns.add('IdInteraction', sql.VarChar(250), { nullable: true });
    table.columns.add('InteractionSection', sql.Int, { nullable: true });
    table.columns.add('InteractionPart', sql.Int, { nullable: true });
    table.columns.add('InteractionFolder', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionType', sql.VarChar(50), { nullable: true });
    table.columns.add('TotalMessages', sql.Int, { nullable: true });
    table.columns.add('InteractionDateStart', sql.DateTime, { nullable: true });
    table.columns.add('InteractionDateEnd', sql.DateTime, { nullable: true });
    table.columns.add('InteractionDuration', sql.Int, { nullable: true });
    table.columns.add('InteractionOrigin', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionManagementResult', sql.VarChar(sql.MAX), { nullable: true });
    table.columns.add('RelativePath', sql.VarChar(500), { nullable: true });
    table.columns.add('FileName', sql.VarChar(200), { nullable: true });
    table.columns.add('InteractionContactId', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionFormat', sql.VarChar(50), { nullable: true });
    table.columns.add('CreatedByUserId', sql.VarChar(50), { nullable: true });
    table.columns.add('InteractionContactAddress', sql.VarChar(300), { nullable: true });
    table.columns.add('InteractionContactName', sql.VarChar(300), { nullable: true });
    logger.info('[EvaluationPlanMSSql::SetInteractionEvaluationPlanTemporaryTable] Records to temporary table InteractionEvaluationPlanTemp: ' + evaluationPlan.channelInteractions.length);
    evaluationPlan.channelInteractions.map(function (i) {
        if (i.ID)
            i.id = i.ID;
        if (i.Direction)
            i.Type = i.Direction;
        if (i.ID && i.Direction) {
            i.Part = 0;
            i.Section = 0;
            i.Channel = 'CALL';
        }
        var managementResult = null;
        if (i.Description) {
            managementResult = '/';
            if (i.Category1)
                managementResult += i.Category1 + '/';
            if (i.Category2)
                managementResult += i.Category2 + '/';
            if (i.Category3)
                managementResult += i.Category3 + '/';
            managementResult += i.Description;
            managementResult = managementResult.split('//').join('/');
        }
        else {
            managementResult = i.dispositionTreePath || null;
        }
        table.rows.add(vcc, evaluationPlan.id, i.Agent ? i.Agent : i.firstTakedAgent || null, i.AgentName ? i.AgentName.trim().replace('  ', ' ') : null, i.Campaign ? i.Campaign : i.campaign || null, i.InteractionId ? i.InteractionId : i.id || null, (i.Section || i.Section === 0) ? i.Section : null, (i.Part || i.Part === 0) ? i.Part : null, i.folder ? i.folder : null, i.Channel ? i.Channel : i.channel || null, i.totalMessages ? i.totalMessages : null, i.StartDate ? moment.utc(i.StartDate).toDate() : moment.utc(i.startDate).toDate() || null, i.EndDate ? moment.utc(i.EndDate).toDate() : moment.utc(i.endDate).toDate() || null, i.DurationTime ? i.DurationTime : i.durationTime || null, i.Type ? (i.Type === 'E' ? 'IN' : 'OUT') : i.initiative || null, managementResult, i.RelativePath ? i.RelativePath : null, i.FileName ? i.FileName : null, i.Number ? i.Number : i.contactName || null, i.Format ? i.Format : i.format || null, evaluationPlan.createdByUserId, i.contactAddress ? i.contactAddress : i.ContactAddress, i.contactName ? i.contactName : i.ContactName);
    });
    return table;
}
function BulkInteractionEvaluationPlanTransaction(vcc, temporaryTable) {
    var transaction = new sql.Transaction(sqlConn);
    var request;
    return transaction.begin()
        .then(function (result) {
        logger.info('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Creating transaction');
        return new sql.Request(transaction);
    })
        .then(function (result) {
        request = result;
        return request.bulk(temporaryTable)
            .then(function (rowCount) {
            logger.info('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Inserted ' + rowCount + ' records to temporary Table');
            return true;
        })
            .catch(function (err) {
            logger.error('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Error on bulk: ' + err.message);
            throw err;
        });
    })
        .then(function (result) {
        return request
            .execute('BulkInteractionEvaluationPlan')
            .then(function (recordsets) {
            logger.info('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Succed Execution');
            var results = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
            var promiseArray = [];
            promiseArray.push(Promise.resolve(results && results.length > 0 ? results[0] : null));
            promiseArray.push(transaction.commit());
            return Promise.all(promiseArray);
        })
            .catch(function (err) {
            logger.error('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Error on: ' + err.message);
            return Promise.reject(err);
        });
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[EvaluationPlanMSSql::BulkInteractionEvaluationPlanTransaction] Parsing error: ' + err.message);
        transaction.rollback();
        return Promise.reject(err);
    });
}
