"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdateNluStatus = exports.SaveCognitiveEngineMigrateStatus = exports.GetNlusUsage = exports.UpdateNluAppId = exports.UpdateVccStatus = exports.UpdateIntentId = exports.UpdateUtteranceId = exports.UpdateEntityId = exports.GetNluIntentUtteranceToExport = exports.DeleteNLU = exports.SaveNLU = exports.GetNLUByAppId = exports.GetNLUByName = exports.GetNLU = exports.GetNLUsStudy = exports.GetUniqueNLUByIntents = exports.GetNLUs = exports.GetInteractionAgents = exports.GetInteractionCampaigns = exports.NLUMSSql = void 0;
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function NLUMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.NLUMSSql = NLUMSSql;
function GetInteractionCampaigns(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetInteractionCampaigns')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractionCampaigns = GetInteractionCampaigns;
function GetInteractionAgents(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetInteractionAgents')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInteractionAgents = GetInteractionAgents;
function GetNLUs(vcc, search_criteria, search_expression, apps, isSystemApp, showInMigrationProcess) {
    if (showInMigrationProcess === void 0) { showInMigrationProcess = false; }
    logger.info('[NLUMSSql::GetNLUs] getting vcc:', vcc, 'search_criteria:', search_criteria, 'search_expression:', search_expression, 'apps:', apps);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('Apps', sql.NVarChar(sql.MAX), apps)
        .input('IsSystemApp', sql.Bit, isSystemApp)
        .input('ShowInMigrationProcess', sql.Bit, showInMigrationProcess)
        .execute('GetNLUs')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLUs] getting recordset: ', recordset && recordset.length ? recordset.length : 0);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUs] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUs = GetNLUs;
function GetUniqueNLUByIntents(vcc, templateId, intents) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('TemplateId', sql.VarChar(100), templateId)
        .input('Intents', sql.NVarChar(sql.MAX), intents)
        .execute('GetUniqueNLUByIntents')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        if (recordset && recordset.length) {
            recordset = recordset.map(function (x) {
                return x.appId;
            });
        }
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetUniqueNLUByIntents] getting errorMsg:', errorMsg, 'vcc:', vcc, 'intents:', intents);
        return Promise.reject(err);
    });
}
exports.GetUniqueNLUByIntents = GetUniqueNLUByIntents;
function GetNLUsStudy(vcc, search_criteria, search_expression, names) {
    logger.info('[NLUMSSql::GetNLUsStudy] getting vcc:', vcc, 'search_criteria:', search_criteria, 'search_expression:', search_expression, 'names:', names);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .input('Names', sql.NVarChar(sql.MAX), names)
        .execute('GetNLUsStudy')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::GetNLUsStudy] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLUsStudy] getting recordset:', recordset);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUsStudy] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUsStudy = GetNLUsStudy;
function GetNLU(vcc, name) {
    logger.info('[NLUMSSql::GetNLU] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetNLU')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::GetNLU] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLU] getting recordset:', recordset);
        var nlu = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::GetNLU] getting nlu:', nlu);
        return Promise.resolve(nlu);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLU] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLU = GetNLU;
function GetNLUByName(vcc, name) {
    logger.info('[NLUMSSql::GetNLUByName] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('GetNLU')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::GetNLUByName] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::GetNLUByName] getting recordset:', recordset);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUByName] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUByName = GetNLUByName;
function GetNLUByAppId(vcc, appId) {
    logger.info('[NLUMSSql::GetNLUByAppId] getting vcc:', vcc, ', appId:', appId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .execute('GetNLUByAppId')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::GetNLUByAppId] getting recordsets: ', recordsets.length);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var nlu = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(nlu);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNLUByAppId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNLUByAppId = GetNLUByAppId;
function SaveNLU(vcc, name, culture, description, appId, subscriptionKey, isSystemApp, needTrain, needPublish, isPublished, lastPublishedDate, isTrained, lastTrainedDate, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    logger.info('[NLUMSSql::SaveNLU] getting vcc:', vcc, 'name:', name, 'culture:', culture, 'description:', description, 'appId:', appId, 'subscriptionKey:', subscriptionKey, 'createdDate:', createdDate, 'createdByUserId:', createdByUserId, 'lastModifiedDate:', lastModifiedDate, 'lastModifiedByUserId:', lastModifiedByUserId);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .input('Culture', sql.VarChar(50), culture)
        .input('Description', sql.VarChar(50), description)
        .input('AppId', sql.VarChar(100), appId)
        .input('SubscriptionKey', sql.VarChar(50), subscriptionKey)
        .input('IsSystemApp', sql.Bit, isSystemApp)
        .input('NeedTrain', sql.Bit, needTrain)
        .input('NeedPublish', sql.Bit, needPublish)
        .input('IsPublished', sql.Bit, isPublished)
        .input('LastPublishedDate', sql.DateTime, lastPublishedDate)
        .input('IsTrained', sql.Bit, isTrained)
        .input('LastTrainedDate', sql.DateTime, lastTrainedDate)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveNLU')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::SaveNLU] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::SaveNLU] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::SaveNLU] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::SaveNLU] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveNLU = SaveNLU;
function DeleteNLU(vcc, name) {
    logger.info('[NLUMSSql::DeleteNLU] getting vcc:', vcc, 'name:', name);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Name', sql.VarChar(50), name)
        .execute('DeleteNLU')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::DeleteNLU] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::DeleteNLU] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::DeleteNLU] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::SaveNLU] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteNLU = DeleteNLU;
function GetNluIntentUtteranceToExport(vcc, listMode) {
    if (listMode === void 0) { listMode = false; }
    logger.info('[NLUMSSql::GetNluIntentUtteranceToExport] getting vcc:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('ListMode', sql.Bit, listMode)
        .execute('GetNluIntentUtteranceToExport')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::GetNluIntentUtteranceToExport] getting recordsets: ', recordsets.length);
        var response = {};
        if (recordsets.length) {
            if (listMode) {
                response.nlus = recordsets[0];
                response.intents = recordsets[1];
                response.utterances = recordsets[2];
                response.entities = recordsets[3];
            }
            else {
                response = {
                    quantity: recordsets[0][0],
                    usage: recordsets[1][0]
                };
            }
        }
        return Promise.resolve(response);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNluIntentUtteranceToExport] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNluIntentUtteranceToExport = GetNluIntentUtteranceToExport;
function UpdateEntityId(vcc, appId, entityId, newEntityId) {
    logger.info('[NLUMSSql::UpdateEntityId] getting UpdateEntityId:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('EntityId', sql.VarChar(50), entityId)
        .input('NewEntityId', sql.VarChar(50), newEntityId)
        .execute('UpdateEntityId')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::UpdateEntityId] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateEntityId] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateEntityId] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateEntityId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateEntityId = UpdateEntityId;
function UpdateUtteranceId(vcc, appId, intentId, utteranceId, newUtteranceId) {
    logger.info('[NLUMSSql::UpdateUtteranceId] getting UpdateUtteranceId:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('UtteranceId', sql.VarChar(50), utteranceId)
        .input('NewUtteranceId', sql.VarChar(50), newUtteranceId)
        .execute('UpdateUtteranceId')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::UpdateUtteranceId] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateUtteranceId] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateUtteranceId] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateUtteranceId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateUtteranceId = UpdateUtteranceId;
function UpdateIntentId(vcc, appId, intentId, newIntentId) {
    logger.info('[NLUMSSql::UpdateIntentId] getting vcc:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('IntentId', sql.VarChar(50), intentId)
        .input('NewIntentId', sql.VarChar(50), newIntentId)
        .execute('UpdateIntentId')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::UpdateIntentId] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateIntentId] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateIntentId] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateIntentId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateIntentId = UpdateIntentId;
function UpdateVccStatus(vcc, isMigrating) {
    if (isMigrating === void 0) { isMigrating = false; }
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('IsMigrating', sql.Bit, isMigrating)
        .execute('UpdateVccStatus')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::UpdateNluAppId] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateNluAppId] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateNluAppId] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateNluAppId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateVccStatus = UpdateVccStatus;
function UpdateNluAppId(vcc, appId, newAppId, culture, cognitiveEngine) {
    logger.info('[NLUMSSql::UpdateNluAppId] getting vcc:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('NewAppId', sql.VarChar(50), newAppId)
        .input('Culture', sql.VarChar(10), culture)
        .input('CognitiveEngine', sql.VarChar(50), cognitiveEngine)
        .execute('UpdateNluAppId')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::UpdateNluAppId] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateNluAppId] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateNluAppId] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateNluAppId] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateNluAppId = UpdateNluAppId;
function GetNlusUsage(vcc, nluNames) {
    logger.info('[NLUMSSql::GetNlusUsage] getting vcc:', vcc);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('NluNames', sql.VarChar(sql.MAX), nluNames)
        .execute('GetNlusUsage')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::GetNlusUsage] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::GetNlusUsage] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::GetNlusUsage] getting output:', output);
        return Promise.resolve(output && output.inUse == true);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::GetNlusUsage] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.GetNlusUsage = GetNlusUsage;
function SaveCognitiveEngineMigrateStatus(vcc, model, key, status, statusCode) {
    logger.info('[NLUMSSql::SaveCognitiveEngineMigrateStatus] Save for ', key);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Model', sql.VarChar(50), model)
        .input('Key', sql.VarChar(100), key)
        .input('Status', sql.Int, status)
        .input('StatusCode', sql.VarChar(50), statusCode)
        .execute('SaveCognitiveEngineMigrateStatus')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::SaveCognitiveEngineMigrateStatus] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::SaveCognitiveEngineMigrateStatus] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::SaveCognitiveEngineMigrateStatus] getting output:', output);
        return Promise.resolve(output && output.result == 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::SaveCognitiveEngineMigrateStatus] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveCognitiveEngineMigrateStatus = SaveCognitiveEngineMigrateStatus;
function UpdateNluStatus(vcc, appId, newStatus) {
    logger.info('[NLUMSSql::UpdateNluStatus]', vcc, appId, newStatus);
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('AppId', sql.VarChar(50), appId)
        .input('NewStatus', sql.VarChar(100), newStatus)
        .execute('UpdateNLUStatus')
        .then(function (recordsets) {
        logger.info('[NLUMSSql::UpdateNluStatus] getting recordsets:', recordsets);
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        logger.info('[NLUMSSql::UpdateNluStatus] getting recordset:', recordset);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        logger.info('[NLUMSSql::UpdateNluStatus] getting output:', output);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error('[NLUMSSql::UpdateNluStatus] getting errorMsg:', errorMsg);
        return Promise.reject(err);
    });
}
exports.UpdateNluStatus = UpdateNluStatus;
