"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CleanerImplementation = void 0;
var log4js = require("log4js");
var config_1 = require("../config/config");
var promises_shared_1 = require("../shared/promises.shared");
var redis_service_1 = require("../db/redis.service");
var mssql_service_1 = require("../db/mssql.service");
var file_manager_1 = require("../shared/file-manager");
var io_model_1 = require("../models/io.model");
var io_request_1 = require("../io/io.request");
var batch_schedule_implementation_1 = require("./batch-schedule.implementation");
var logger = log4js.getLogger('Task');
var CleanerImplementation = (function () {
    function CleanerImplementation() {
    }
    CleanerImplementation.prototype.Execute = function (executionCode) {
        var _this = this;
        var totalProgress = [];
        var progress = [];
        var errorProgress = [];
        var summaryProgress = [];
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            return _this.CleanReadyProcessI6Updates(executionCode);
        })
            .then(function (_) {
            return _this.CleanReadyProcessInteractions(executionCode);
        })
            .then(function (_) {
            return _this.CleanReadyStudyCognitiveEvaluations(executionCode);
        })
            .then(function (_) {
            return _this.CleanReadyEvaluationPlanEvaluations(executionCode);
        })
            .then(function (_) {
            return _this.CleanReadyEvaluationPlanTipifications(executionCode);
        })
            .then(function (_) {
            return redis_service_1.Redis.GetTotalProgress();
        }).then(function (result) {
            totalProgress = result;
            return redis_service_1.Redis.GetErrorProgress();
        }).then(function (result) {
            errorProgress = result;
            return redis_service_1.Redis.GetProgress();
        }).then(function (result) {
            progress = result;
            if (totalProgress && totalProgress.length) {
                totalProgress.forEach(function (total) {
                    var currentProgress = progress.find(function (item) { return item.key === total.key; });
                    var currentErrorProgress = errorProgress.find(function (item) { return item.key === total.key; });
                    total.value = total.value ? total.value : 0;
                    currentProgress.value = currentProgress && currentProgress.value ? currentProgress.value : 0;
                    var realProgress = total.value - currentProgress.value;
                    var summaryItem = JSON.parse(JSON.stringify(total));
                    summaryItem['progress'] = realProgress;
                    summaryItem['percentage'] = currentProgress.value <= 0 ? 1 : Math.round((realProgress / total.value) * 100000) / 100000;
                    summaryItem['error'] = currentErrorProgress && currentErrorProgress.value ? currentErrorProgress.value : 0;
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'RealProgress: ', realProgress);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'TotalValue: ', total.value);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'CurrentProgress: ', currentProgress.value);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'CurrentErrorProgress: ', currentErrorProgress && currentErrorProgress.value ? currentErrorProgress.value : 0);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'Progress: ', summaryItem['progress']);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'Percentaje: ', summaryItem['percentage']);
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'Error: ', summaryItem['error']);
                    summaryProgress.push(summaryItem);
                });
            }
            return mssql_service_1.MSSql.BulkProgressTemp(summaryProgress);
        })
            .then(function (_) {
            var schedulesToStart = summaryProgress.filter(function (sp) { return sp.percentage === 1 && [null, 'ability'].includes(sp.type); });
            if (schedulesToStart.length) {
                logger.info("[CleanerImplementation:Execute] executionCode: " + executionCode + " searching schedules for " + schedulesToStart.length + " process/evaluationPlan");
                var batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
                return batchScheduleImplementation.GetNewScheduledBatchExecutions(executionCode);
            }
            return Promise.resolve(true);
        })
            .then(function (_) {
            var promiseArray = [];
            var process = {};
            for (var _i = 0, summaryProgress_1 = summaryProgress; _i < summaryProgress_1.length; _i++) {
                var item = summaryProgress_1[_i];
                var batchName = null;
                switch (item.type) {
                    case 'study':
                        batchName = 'STUDY';
                        break;
                    case 'template':
                        batchName = 'TEMPLATE';
                        break;
                    case 'ability':
                        batchName = 'EVALUATION_PLAN';
                        break;
                    default:
                        batchName = 'TRANSCRIPTION';
                        item.typeId = item.processId;
                        break;
                }
                if (item.percentage === 1) {
                    logger.info('[CleanerImplementation:Execute] executionCode:', executionCode, 'FINALIZED', item);
                    promiseArray.push(redis_service_1.Redis.DeleteTotalProgress(item.key));
                    promiseArray.push(redis_service_1.Redis.DeleteProgress(item.key));
                    promiseArray.push(redis_service_1.Redis.DeleteErrorProgress(item.key));
                    if (['study', 'template'].includes(item.type))
                        promiseArray.push(redis_service_1.Redis.RemoveStudyTemplateThemes(item.vcc, item.typeId));
                    if (['ability'].includes(item.type))
                        promiseArray.push(redis_service_1.Redis.RemoveEvaluationPlanAbilities(item.vcc, item.typeId));
                    if (batchName) {
                        promiseArray.push(redis_service_1.Redis.DeleteBatchRasaModels(batchName.toUpperCase() + '::' + item.typeId));
                    }
                }
                process[item.vcc + "-" + batchName] = {
                    vcc: item.vcc, type: batchName
                };
            }
            if (Object.keys(process).length > 0) {
                var events = [];
                for (var p in process) {
                    events.push(new io_model_1.IOEvent(io_model_1.VCC_EVENT, "UPDATE_" + process[p].type, null, process[p].vcc));
                }
                promiseArray.push(io_request_1.SendIOEvents(events));
            }
            return Promise.all(promiseArray);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation:Execute] executionCode:', executionCode, 'Progress getting error:', errorMessage);
            return Promise.resolve(false);
        }).then(function (_) {
            logger.info('[CleanerImplementation:Execute] executionCode: CleanupTempFile', executionCode, 'getting result:', 'DELETED FILES');
            var promiseArray = [];
            promiseArray.push(file_manager_1.CleanupTempFile(config_1.PATH_FOLDER_TEMP_AUDIO, config_1.PERSISTENCE_AUDIO_TEMP));
            return Promise.all(promiseArray);
        }).then(function (result) {
            logger.info('[CleanerImplementation:Execute] executionCode:', executionCode, 'getting result:', 'TASK FINISHED');
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation:Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    CleanerImplementation.prototype.CleanReadyProcessI6Updates = function (executionCode) {
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            return redis_service_1.Redis.GetReadyI6UpdateInteractions(executionCode);
        })
            .then(function (interactions) {
            logger.info('[CleanerImplementation::CleanReadyProcessI6Updates] executionCode:', executionCode, 'getting: interactions:', (interactions && interactions.length ? interactions.length : 0), 'after Redis.GetReadyProcessI6Update');
            if (interactions && interactions.length) {
                return mssql_service_1.MSSql.BulkProcessI6UpdateTemp(interactions);
            }
            return Promise.resolve(true);
        })
            .then(function (_) {
            var hash = 'I6Update:' + executionCode;
            return redis_service_1.Redis.Del(hash, executionCode);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::CleanReadyProcessI6Updates] executionCode:', executionCode, 'BulkProcessInteractionTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    CleanerImplementation.prototype.CleanReadyProcessInteractions = function (executionCode) {
        var _this = this;
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            return redis_service_1.Redis.GetAllReadyProcessInteraccionStorages(executionCode);
        })
            .then(function (storages) {
            logger.debug("[CleanerImplementation::CleanReadyProcessInteractions] Storages " + storages);
            if (storages && storages.length) {
                var oldStorages = storages.filter(function (s) { return !s.includes(executionCode); });
                if (oldStorages) {
                    logger.info("[CleanerImplementation::CleanReadyProcessInteractions] executionCode: " + executionCode + ", " + oldStorages + " old storages will be taked.");
                }
            }
            var promiseArray = [];
            for (var i = 0; i < 5; i++) {
                promiseArray.push(_this.SaveReadyProcessInteractions(storages[i], executionCode));
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (result.includes(false))
                logger.warn("[CleanerImplementation::CleanReadyProcessInteractions] executionCode: " + executionCode + ", " + result.filter(function (r) { return r === false; }).length + " storages could not be saved.");
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::CleanReadyProcessInteractions] executionCode:', executionCode, 'BulkProcessInteractionTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    CleanerImplementation.prototype.SaveReadyProcessInteractions = function (storage, executionCode) {
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            if (storage !== executionCode)
                logger.info("[CleanerImplementation::SaveReadyProcessInteractions] executionCode: " + executionCode + ", searching old starage from execution " + storage);
            return redis_service_1.Redis.GetReadyProcessInteractions(storage);
        })
            .then(function (interactions) {
            logger.info('[CleanerImplementation::SaveReadyProcessInteractions] executionCode:', executionCode, 'getting: interactions:', (interactions && interactions.length ? interactions.length : 0), 'after Redis.GetReadyProcessInteractions');
            if (interactions && interactions.length) {
                return mssql_service_1.MSSql.BulkProcessInteractionTemp(interactions);
            }
            return Promise.resolve(true);
        })
            .then(function (_) {
            var hash = 'Audios:' + storage;
            return redis_service_1.Redis.Del(hash, executionCode);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::SaveReadyProcessInteracctions] executionCode:', executionCode, 'BulkProcessInteractionTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    CleanerImplementation.prototype.CleanReadyStudyCognitiveEvaluations = function (executionCode) {
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            return redis_service_1.Redis.GetReadyCognitiveInteractions(executionCode);
        })
            .then(function (interactions) {
            logger.info('[CleanerImplementation::CleanReadyStudyCognitiveEvaluations] executionCode:', executionCode, 'getting: interactions:', (interactions && interactions.length ? interactions.length : 0), 'after Redis.GetReadyCognitiveInteractions');
            if (interactions && interactions.length) {
                return mssql_service_1.MSSql.BulkInteractionCognitiveTemp(interactions);
            }
            return Promise.resolve(true);
        })
            .then(function (result) {
            var hash = 'Cognitive:' + executionCode;
            return redis_service_1.Redis.Del(hash, executionCode);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::CleanReadyStudyCognitiveEvaluations] executionCode:', executionCode, 'BulkInteractionCognitiveTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    CleanerImplementation.prototype.CleanReadyEvaluationPlanEvaluations = function (executionCode) {
        var _this = this;
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            return redis_service_1.Redis.GetAllReadyCognitiveAbilityStorages(executionCode);
        })
            .then(function (storages) {
            logger.debug("[CleanerImplementation::CleanReadyEvaluationPlanEvaluations] Storages " + storages);
            if (storages === null || storages === void 0 ? void 0 : storages.length) {
                var oldStorages = storages.filter(function (s) { return !s.includes(executionCode); });
                if (oldStorages)
                    logger.info("[CleanerImplementation::CleanReadyEvaluationPlanEvaluations] executionCode: " + executionCode + ", " + oldStorages + " old storages will be taked.");
            }
            var promiseArray = [];
            for (var i = 0; i < 5; i++) {
                promiseArray.push(_this.SaveReadyEvaluationPlanEvaluations(storages[i], executionCode));
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (result.includes(false))
                logger.warn("[CleanerImplementation::CleanReadyEvaluationPlanEvaluations] executionCode: " + executionCode + ", " + result.filter(function (r) { return r === false; }).length + " storages could not be saved.");
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::CleanReadyEvaluationPlanEvaluations] executionCode:', executionCode, 'BulkAbilityCognitiveTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    CleanerImplementation.prototype.SaveReadyEvaluationPlanEvaluations = function (storage, executionCode) {
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            if (storage !== executionCode)
                logger.info("[CleanerImplementation::SaveReadyEvaluationPlanEvaluations] executionCode: " + executionCode + ", searching old starage from execution " + storage);
            return redis_service_1.Redis.GetReadyCognitiveAbilities(storage);
        })
            .then(function (interactions) {
            logger.info('[CleanerImplementation::SaveReadyEvaluationPlanEvaluations] executionCode:', executionCode, 'getting: interactions:', ((interactions === null || interactions === void 0 ? void 0 : interactions.length) ? interactions.length : 0), 'after Redis.GetReadyTipificationInteractions');
            if (interactions === null || interactions === void 0 ? void 0 : interactions.length) {
                return mssql_service_1.MSSql.BulkAbilityCognitiveTemp(interactions);
            }
            return Promise.resolve(true);
        })
            .then(function (_) {
            var hash = 'AbilityCognitive:' + storage;
            return redis_service_1.Redis.Del(hash, executionCode);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::SaveReadyEvaluationPlanEvaluations] executionCode:', executionCode, 'BulkAbilityCognitiveTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    CleanerImplementation.prototype.CleanReadyEvaluationPlanTipifications = function (executionCode) {
        var _this = this;
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            return redis_service_1.Redis.GetAllReadyTipificationInteractionStorages(executionCode);
        })
            .then(function (storages) {
            logger.debug("[CleanerImplementation::CleanReadyEvaluationPlanTipifications] Storages " + storages);
            var promiseArray = [];
            if (storages === null || storages === void 0 ? void 0 : storages.length) {
                var oldStorages = storages.filter(function (s) { return !s.includes(executionCode); }).length;
                if (oldStorages)
                    logger.info("[CleanerImplementation::CleanReadyEvaluationPlanTipifications] executionCode: " + executionCode + ", " + oldStorages + " old storages will be taked.");
                for (var i = 0; i < 5; i++) {
                    if (storages[i])
                        promiseArray.push(_this.SaveReadyEvaluationPlanTipifications(storages[i], executionCode));
                }
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            if (result.includes(false))
                logger.warn("[CleanerImplementation::CleanReadyEvaluationPlanTipifications] executionCode: " + executionCode + ", " + result.filter(function (r) { return r === false; }).length + " storages could not be saved.");
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::CleanReadyEvaluationPlanTipifications] executionCode:', executionCode, 'BulkProcessInteractionTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    CleanerImplementation.prototype.SaveReadyEvaluationPlanTipifications = function (storage, executionCode) {
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            if (storage !== executionCode)
                logger.info("[CleanerImplementation::SaveReadyEvaluationPlanTipifications] executionCode: " + executionCode + ", searching old starage from execution " + storage);
            return redis_service_1.Redis.GetReadyTipificationInteractions(storage);
        })
            .then(function (interactions) {
            logger.info('[CleanerImplementation::SaveReadyEvaluationPlanTipifications] executionCode:', executionCode, 'getting: interactions:', ((interactions === null || interactions === void 0 ? void 0 : interactions.length) ? interactions.length : 0), 'after Redis.GetReadyTipificationInteractions');
            if (interactions === null || interactions === void 0 ? void 0 : interactions.length) {
                return mssql_service_1.MSSql.BulkTipificationInteractionTemp(interactions);
            }
            return Promise.resolve(true);
        })
            .then(function (_) {
            var hash = 'ReadyTipificationInteraction:' + storage;
            return redis_service_1.Redis.Del(hash, executionCode);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::SaveReadyEvaluationPlanTipifications] executionCode:', executionCode, 'TipificationInteractionTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    return CleanerImplementation;
}());
exports.CleanerImplementation = CleanerImplementation;
