"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SentimentAnalysisImplementation = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var config_1 = require("../config/config");
var redis_service_1 = require("../db/redis.service");
var file_manager_1 = require("../shared/file-manager");
var promises_shared_1 = require("../shared/promises.shared");
var cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
var billing_implementation_1 = require("./billing.implementation");
var diarization_analysis_implementation_1 = require("../implementation/diarization-analysis.implementation");
var logger = log4js.getLogger('Task');
var loggerService = log4js.getLogger('ServerScripts');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'SentimentAnalysisImplementation';
var SentimentAnalysisImplementation = (function () {
    function SentimentAnalysisImplementation() {
    }
    SentimentAnalysisImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_SENTIMENT_ANALYSIS;
        var executeResult = true;
        return promises_shared_1.SleepPromise(config_1.SLEEP_SENTIMENT_ANALYSIS)
            .then(function (result) {
            return redis_service_1.Redis.LRange('SentimentAnalysis', 0, ratio - 1);
        }).then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(_this.SentimentAnalysisExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('SentimentAnalysis', 0, ratio - 1);
        }).then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        }).then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SentimentAnalysisImplementation::Execute] executionCode:', executionCode, 'getting error results:', errorMessage);
            return Promise.resolve(false);
        });
    };
    SentimentAnalysisImplementation.prototype.SentimentAnalysisExecute = function (executionCode) {
        var _this = this;
        var currentBlockPromise = 'BLOCK_BEFORE_PROMISE';
        var storage = '';
        var currentQueueId = '';
        var processInteraction;
        var isCancel = false;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        var batchTypeName = '';
        var batchTypeId = '';
        return redis_service_1.Redis.RPopLPush('SentimentAnalysis', 'SentimentAnalysisExecute')
            .then(function (queueId) {
            currentBlockPromise = 'BLOCK_TO_GET_PROCESS_INTERACTION';
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then(function (processInteractionResult) {
            logger.debug('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Interaction: ', processInteractionResult);
            currentBlockPromise = 'BLOCK_TO_LAUNCH_MAIN_FUNCTION';
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                if (processInteraction.statusDescription === 'pendingSentimentAnalysis') {
                    return _this.EvaluateSentiments(processInteraction, currentQueueId, executionCode);
                }
                return null;
            }
            else {
                if (currentQueueId)
                    logger.warn('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return null;
            }
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_UPDATE_INTERACTION';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return null;
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_LAUNCH_FINISHING_FUNCTION';
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result
                && processInteraction
                && processInteraction.status === 5
                && processInteraction.statusDescription === 'pendingAnalysis') {
                logger.info('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticAnalysis', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SentimentAnalysisExecute', currentQueueId));
            }
            else if (result
                && processInteraction
                && (processInteraction.status === 4
                    ||
                        (processInteraction.status === 5 && processInteraction.statusDescription !== 'pendingSentimentAnalysis'))
                && processInteraction.retry < config_1.RETRY_SENTIMENT_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('SentimentAnalysis', currentQueueId, config_1.RATE_RECURSIVE_SENTIMENT_ANALYSIS * config_1.RATIO_SENTIMENT_ANALYSIS * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SentimentAnalysisExecute', currentQueueId));
            }
            else if (result
                && processInteraction
                && processInteraction.status === -5
                && processInteraction.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SentimentAnalysisExecute', currentQueueId));
            }
            else if (processInteraction && processInteraction.retry >= config_1.RETRY_SENTIMENT_ANALYSIS) {
                logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SentimentAnalysisExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, batchTypeName, batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SentimentAnalysisExecute', currentQueueId));
            }
            else {
                logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.resolve(true);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_UPDATE_REDIS';
            var promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_SENTIMENT_ANALYSIS) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processId, processId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_FINISH_EXECUTION';
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    SentimentAnalysisImplementation.prototype.EvaluateSentiments = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var path = '';
        var jsonContent = null;
        var cognitiveService;
        var cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
        var phrases = null;
        var agentResults = [];
        var clientResults = [];
        var agentEvaluationResult = [];
        var clientEvaluationResult = [];
        var averageResults = [];
        var averageResultsDB = [];
        var intentsDB;
        var needInvertChannels = false;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            var regex = /\.(mp3|wav)$/;
            var jsonFileName = interaction.fileName.replace(regex, '.json');
            path = config_1.TRANSCRIPTION_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return file_manager_1.ReadFileContent(path, 'utf-8');
        }).then(function (result) {
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
                var diarizationAnalysis = new diarization_analysis_implementation_1.DiarizationAnalysisImplementation();
                phrases = diarizationAnalysis.GeneratePhrases(jsonContent.results, config_1.COGNITIVE_SERVICES_RASA_CHARS_PER_PHRASE);
                logger.debug('[SentimentAnalysisImplementation::EvaluateSentiments] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting phrases:', phrases);
            }
            catch (ex) {
                jsonContent = '';
                throw 'SERVER_ERROR_GET_JSON_FAILED';
            }
            return Promise.resolve(phrases);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_PHRASES_NOT_FOUND';
            return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_COGNITIVE_SERVICE';
            cognitiveService = result;
            var promiseArray = [];
            phrases.agent.map(function (ap, i) {
                var appId = interaction.vcc + '::sentiment-' + interaction.language;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        return resolve(cognitiveAnalysis.ModelQuery(cognitiveService, null, null, appId, ap, appId, executionCode, currentQueueId));
                    }, i * 1000);
                }));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_AGENT_EVALUATION_FAILED';
            logger.debug('[SentimentAnalysisImplementation::EvaluateSentiments] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting agent evaluation result:', result);
            agentEvaluationResult = result;
            var promiseArray = [];
            phrases.client.map(function (ap, i) {
                var appId = interaction.vcc + '::sentiment-' + interaction.language;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        return resolve(cognitiveAnalysis.ModelQuery(cognitiveService, null, null, appId, ap, appId, executionCode, currentQueueId));
                    }, i * 1000);
                }));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_CLIENT_EVALUATION_FAILED';
            logger.debug('[SentimentAnalysisImplementation::EvaluateSentiments] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting client evaluation result:', result);
            clientEvaluationResult = result;
            agentEvaluationResult.map(function (aer, i) {
                agentResults.push({
                    phrase: phrases.agent[i],
                    intentScores: aer.intents
                });
            });
            clientEvaluationResult.map(function (cer, i) {
                clientResults.push({
                    phrase: phrases.client[i],
                    intentScores: cer.intents
                });
            });
            return mssql_service_1.MSSql.GetIntents(interaction.vcc, interaction.vcc + '::sentiment-' + interaction.language, null, null, null);
        }).then(function (result) {
            intentsDB = result;
            var agentPhrasesNumber = agentResults.length;
            var clientPhrasesNumber = clientResults.length;
            var generalPhrasesNumber = agentResults.length + clientResults.length;
            var generalResults = agentResults.concat(clientResults);
            var agentAverageResults = [];
            var clientAverageResults = [];
            var generalAverageResults = [];
            intentsDB.forEach(function (intentDB) {
                var sumAgentResults = 0;
                var sumClientResults = 0;
                var sumGeneralResults = 0;
                var saveScore = 0;
                agentResults.forEach(function (agentr) {
                    var found = agentr.intentScores ? agentr.intentScores.find(function (intentScore) { return intentScore.intent === intentDB.intentId; }) : null;
                    if (found)
                        sumAgentResults = sumAgentResults + found.score;
                });
                agentAverageResults.push({
                    intentResult: intentDB.intentId,
                    averageResult: agentPhrasesNumber > 0 ? (sumAgentResults / agentPhrasesNumber).toFixed(5) : 0
                });
                clientResults.forEach(function (clientr) {
                    var found = clientr.intentScores ? clientr.intentScores.find(function (intentScore) { return intentScore.intent === intentDB.intentId; }) : null;
                    if (found)
                        sumClientResults = sumClientResults + found.score;
                });
                clientAverageResults.push({
                    intentResult: intentDB.intentId,
                    averageResult: clientPhrasesNumber > 0 ? (sumClientResults / clientPhrasesNumber).toFixed(5) : 0
                });
                generalResults.forEach(function (allr) {
                    var found = allr.intentScores ? allr.intentScores.find(function (intentScore) { return intentScore.intent === intentDB.intentId; }) : null;
                    if (found)
                        sumGeneralResults = sumGeneralResults + found.score;
                });
                generalAverageResults.push({
                    intentResult: intentDB.intentId,
                    averageResult: generalPhrasesNumber > 0 ? (sumGeneralResults / generalPhrasesNumber).toFixed(5) : 0
                });
            });
            averageResults.push({ analysisType: 'agent', averageResults: _this.GetMax(agentAverageResults) });
            averageResults.push({ analysisType: 'client', averageResults: _this.GetMax(clientAverageResults) });
            averageResults.push({ analysisType: 'general', averageResults: _this.GetMax(generalAverageResults) });
            var totalScore = 0;
            averageResults.forEach(function (ar) {
                ar.averageResults.forEach(function (are) {
                    var saveScore = parseFloat(are.averageResult);
                    if (saveScore > 0) {
                        totalScore = totalScore + saveScore;
                        mssql_service_1.MSSql.SaveSentimentAnalysisResult(interaction.vcc, interaction.id, interaction.section ? interaction.section : 0, interaction.part ? interaction.part : 0, interaction.interactionType, 0, ar.analysisType, are.intentId, interaction.vcc + '::sentiment-' + interaction.language, saveScore, null, 'admin', null, 'admin', interaction.endDate ? interaction.endDate.toString() : null, are.status);
                    }
                });
            });
            if (totalScore > 0)
                return true;
            return false;
        }).then(function (result) {
            if (!result)
                return Promise.resolve('ERROR');
            return Promise.resolve('OK');
        })
            .then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 5;
                interaction['statusDescription'] = 'pendingAnalysis';
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[SentimentAnalysisImplementation::EvaluateSentiments] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -5;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'EvaluateSentiments';
                interaction['errorMessage'] = 'SENTIMENT_MODEL_WAS_NOT_TRAINED_CORRECTLY';
                interaction['errorDetail'] = 'SENTIMENT_MODEL_WAS_NOT_TRAINED_CORRECTLY';
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SentimentAnalysisImplementation::EvaluateSentiments] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -5;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'EvaluateSentiments';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    };
    SentimentAnalysisImplementation.prototype.SaveSentimentResults = function (interaction, jsonContent, currentQueueId, executionCode) {
        var sentimentFolder = config_1.SENTIMENT_FOLDER + '/' + interaction.vcc;
        var regex = /\.(mp3|wav|json)$/;
        var jsonFileName = interaction.fileName.replace(regex, '.json');
        var jsonFilePath = null;
        jsonFilePath = sentimentFolder;
        jsonFilePath += '/';
        jsonFilePath += jsonFileName;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return file_manager_1.CreateFolder(sentimentFolder);
        })
            .then(function (result) {
            logger.info('[SentimentAnalysisImplementation::SaveSentimentResults] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'saving results on:', jsonFilePath);
            return file_manager_1.WriteFileContent(jsonFilePath, JSON.stringify(jsonContent), 'utf-8');
        })
            .then(function (result) {
            return Promise.resolve(true);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[SentimentAnalysisImplementation::SaveSentimentResults] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.reject(errorMessage);
        });
    };
    SentimentAnalysisImplementation.prototype.GetSentimentResultsFromJsonFile = function (jsonFileName, vcc) {
        var path = '';
        var jsonContent = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            path = config_1.SENTIMENT_FOLDER;
            path += '/';
            path += vcc;
            path += '/';
            path += jsonFileName;
            return (file_manager_1.ExistInFileSystem(path));
        }).then(function (result) {
            if (!result)
                return null;
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then(function (result) {
            jsonContent = typeof result === 'string' && result !== null ? JSON.parse(result) : result;
            if (jsonContent !== null)
                return Promise.resolve(jsonContent.AverageResults);
            return Promise.resolve(null);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[SentimentAnalysisImplementation::GetSentimentResults] executionCode:', 'getting error:', errorMessage);
            return Promise.reject(errorMessage);
        });
    };
    SentimentAnalysisImplementation.prototype.GetSentimentResults = function (jsonFileName, vcc, interactionType) {
        jsonFileName = jsonFileName.replace('.json', '');
        var interactionId = '';
        var section = 0;
        var part = 0;
        var endDate = '';
        if (interactionType === 'CALL') {
            var lastFour = jsonFileName.substr(-4);
            section = Number(lastFour.split('_')[1]);
            part = Number(lastFour.split('_')[2]);
            interactionId = jsonFileName.replace(lastFour, '');
        }
        else {
            interactionId = jsonFileName.split('_')[3];
            endDate = jsonFileName.split('_')[2];
        }
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetSentimentAnalysisResult(vcc, interactionId, section, part, endDate);
        })
            .then(function (result) {
            if (result !== null)
                return Promise.resolve(result);
            return Promise.resolve(null);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[SentimentAnalysisImplementation::GetSentimentResults] executionCode:', 'getting error:', errorMessage);
            return Promise.reject(errorMessage);
        });
    };
    SentimentAnalysisImplementation.prototype.GetMax = function (resultsScores) {
        var maxValues = [];
        var highest = 0;
        var intentIdMax = '';
        var tmp = 0;
        var newResultScores = [];
        resultsScores.map(function (element) {
            tmp = Number(element.averageResult);
            if (tmp > highest) {
                highest = tmp;
                intentIdMax = element.intentResult;
            }
        });
        maxValues.push({
            intentIdMax: intentIdMax,
            averageMax: highest
        });
        resultsScores.map(function (score) {
            var found = maxValues.find(function (maxval) { return maxval.intentIdMax === score.intentResult; });
            if (found) {
                newResultScores.push({
                    intentId: score.intentResult,
                    averageResult: score.averageResult,
                    status: "max"
                });
            }
            else {
                newResultScores.push({
                    intentId: score.intentResult,
                    averageResult: score.averageResult,
                    status: "normal"
                });
            }
        });
        return newResultScores;
    };
    return SentimentAnalysisImplementation;
}());
exports.SentimentAnalysisImplementation = SentimentAnalysisImplementation;
