"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerateNewBatcScheduleExecutions = exports.SaveEvaluationPlanBatchSchedule = exports.SaveTemplateBatchSchedule = exports.SaveProcessBatchSchedule = exports.BatchScheduleService = void 0;
var log4js = require("log4js");
var moment = require("moment");
var mssql_service_1 = require("../db/mssql.service");
var schedule_model_1 = require("../models/schedule.model");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var batch_schedule_implementation_1 = require("../implementation/batch-schedule.implementation");
var template_service_1 = require("./template.service");
var arrays_shared_1 = require("../shared/arrays.shared");
var logger = log4js.getLogger('ServerScripts');
function BatchScheduleService(app) {
    app.post('/api/schedule/get_schedule', function (req, res, next) {
        logger.info('[BatchScheduleService::get_schedule] Received ' + req.method + ' methos on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var bodyData = (req.body || {});
        var batchScheduleId = bodyData.scheduleId;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchScheduleById(vcc, batchScheduleId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_schedule] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/schedule/get_transcription_schedules', function (req, res, next) {
        logger.info('[BatchScheduleService::get_transcription_schedules] Received ' + req.method + ' methos on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var bodyData = (req.body || {});
        var searchCriteria = bodyData.searchCriteria;
        var searchExpression = bodyData.searchExpression;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchSchedulesSummary(vcc, searchCriteria, searchExpression, 'TRANSCRIPTION');
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_transcription_schedules] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/schedule/get_template_schedules', function (req, res, next) {
        logger.info('[BatchScheduleService::get_template_schedules] Received ' + req.method + ' methos on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var bodyData = (req.body || {});
        var searchCriteria = bodyData.searchCriteria;
        var searchExpression = bodyData.searchExpression;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchSchedulesSummary(vcc, searchCriteria, searchExpression, 'TEMPLATE');
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_template_schedules] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/schedule/get_schedule_executions', function (req, res, next) {
        logger.info('[BatchScheduleService::get_schedule_executions] Received ' + req.method + ' methos on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var bodyData = (req.body || {});
        var batchScheduleId = bodyData.scheduleId;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchScheduleExecutionsByBatchType(vcc, batchScheduleId, null);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_schedule_executions] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/schedule/save_schedule_status', function (req, res, next) {
        logger.info('[BatchScheduleService::save_schedule_status] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var bodyData = (req.body || {});
        var batchScheduleId = bodyData.scheduleId;
        var newStatus = bodyData.status;
        var scheduleData = null;
        var saveResult = false;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchScheduleById(vcc, batchScheduleId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_BATCH_SCHEDULE_NOT_FOUND';
            scheduleData = result;
            switch (newStatus) {
                case 'PAUSE':
                    scheduleData.status = 'PAUSED';
                    break;
                case 'RESUME':
                    scheduleData.status = 'RUNNING';
                    break;
            }
            return mssql_service_1.MSSql.SaveBatchSchedule(vcc, scheduleData, userId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_BATCH_SCHEDULE_STATUS_UPDATE_FAILED';
            saveResult = result;
            var promiseArray = [];
            switch (newStatus) {
                case 'PAUSE':
                    promiseArray.push(DeleteBatchsScheduleExecutionsData(vcc, scheduleData));
                    break;
                case 'RESUME':
                    promiseArray.push(CreateBatchScheduleExecutionsData(vcc, scheduleData));
                    break;
            }
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            if (res.headersSent)
                return;
            res.json({ status: saveResult, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'STUDY_SERVER_ERROR_ON_SAVE'), data: saveResult });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::save_schedule_status] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/schedule/delete_schedule', function (req, res, next) {
        logger.info('[BatchScheduleService::delete_schedule] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var batchSchedule = req.body.schedule;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!batchSchedule)
                throw 'SERVER_ERROR_MISSING_SCHEDULE_ID';
            return mssql_service_1.MSSql.DeleteBatchSchedule(vcc, batchSchedule.batchType, batchSchedule.id);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_SCHEDULE_DELETE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_SCHEDULE_DELETE_FAILED'), data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::delete_schedule] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/schedule/get_scheduled_template_results_summary', function (req, res, next) {
        logger.info('[BatchScheduleService::get_scheduled_template_results_summary] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var templateColors = [
            { color: '#FF864B', font: '#FFFFFF' },
            { color: '#155979', font: '#FFFFFF' },
            { color: '#82B7E8', font: '#202121' },
            { color: '#A45C6F', font: '#FFFFFF' },
            { color: '#E57B80', font: '#FFFFFF' },
            { color: '#FC9C51', font: '#FFFFFF' },
            { color: '#C56B77', font: '#FFFFFF' },
            { color: '#FF864B', font: '#FFFFFF' },
            { color: '#155979', font: '#FFFFFF' },
            { color: '#82B7E8', font: '#202121' },
            { color: '#A45C6F', font: '#FFFFFF' },
            { color: '#E57B80', font: '#FFFFFF' },
            { color: '#FC9C51', font: '#FFFFFF' },
            { color: '#C56B77', font: '#FFFFFF' }
        ];
        var filters = req.body || {};
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchScheduledTemplateResults(vcc, filters.scheduledTemplates, {
                startDate: moment(filters.dateIni).toDate(),
                endDate: moment(filters.dateFin).toDate()
            });
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_BATCH_SCHEDULED_TEMPLATE_RESULTS_NOT_FOUND';
            var resultSummary = result;
            var latestResults = result.filter(function (r) { return r.latestOcurrencies === 1; });
            var data = {
                children: [],
                total: 0
            };
            var scheduledTemplates = arrays_shared_1.GetUniqueFromObjectArray(resultSummary, 'batchScheduleId');
            if (scheduledTemplates.length === 1) {
                data.batchScheduledTemplate = scheduledTemplates[0].batchScheduleId;
                data.level = 1;
                data.items = GetBatchScheduledTemplateData(resultSummary, latestResults);
                data.items.map(function (c) {
                    c.isFirstLevel = true;
                    c.color = data.color;
                });
                data.children = data.items;
                data.total = arrays_shared_1.GetSumFromObjectArray(data.children, 'total');
                data.latestResults = latestResults.filter(function (lr) { return lr.batchScheduleId === scheduledTemplates[0].batchScheduleId; }).length || 0;
            }
            else {
                scheduledTemplates.map(function (st, i) {
                    var currentTemplate = {
                        batchScheduledTemplate: st.batchScheduleId,
                        level: 1,
                        isFirstLevel: true,
                        isFinalLevel: false,
                        name: st.batchScheduleId,
                        items: GetBatchScheduledTemplateData(resultSummary.filter(function (r) { return r.batchScheduleId === st.batchScheduleId; }), latestResults)
                    };
                    currentTemplate.value = arrays_shared_1.GetSumFromObjectArray(currentTemplate.items, 'total');
                    currentTemplate.total = currentTemplate.value;
                    currentTemplate.latestResults = latestResults.filter(function (lr) { return lr.batchScheduleId === st.batchScheduleId; }).length || 0;
                    data.children.push(currentTemplate);
                    data.total += currentTemplate.total;
                });
            }
            data.children = arrays_shared_1.SortObjectArrayByValue(data.children, 'total', 'D');
            if (data.batchScheduledTemplate) {
                data.color = templateColors[0].color;
                data.items.map(function (c, i) { return c.color = data.color; });
            }
            else {
                data.children.map(function (c, i) { return c.color = templateColors[(i % templateColors.length)].color; });
            }
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: data });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_scheduled_template_results_summary] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/schedule/get_scheduled_template_item_results_detailed', function (req, res, next) {
        logger.info('[BatchScheduleService::get_scheduled_template_item_results_detailed] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var filters = req.body || {};
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchScheduledTemplateItemResults(vcc, {
                batchScheduledId: filters.batchScheduledTemplateId,
                startDate: moment(filters.startDate).toDate(),
                endDate: moment(filters.endDate).toDate(),
                category1: filters.category1,
                category2: filters.category2,
                description: filters.description
            });
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_scheduled_template_results_summary] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.BatchScheduleService = BatchScheduleService;
function SaveBatchSchedule(vcc, scheduleData, userId) {
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return mssql_service_1.MSSql.GetBatchScheduleById(vcc, scheduleData.id);
    })
        .then(function (result) {
        if (result)
            throw 'SERVER_ERROR_BATCH_SCHEDULE_ALREADY_EXIST';
        scheduleData.id = scheduleData.id.toLowerCase();
        return mssql_service_1.MSSql.SaveBatchSchedule(vcc, scheduleData, userId);
    })
        .then(function (_) {
        if (scheduleData.batchType === 'TRANSCRIPTION')
            return CreateBatchScheduleExecutionsData(vcc, scheduleData);
        else
            return Promise.resolve(true);
    })
        .then(function (_) {
        logger.info('[BatchScheduleService::SaveBatchSchedule] Finishing function');
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[BatchScheduleService::SaveBatchSchedule] Function failed');
        return Promise.reject(err);
    });
}
function CreateBatchScheduleExecutionsData(vcc, scheduleData) {
    var executionCode = moment().toDate().toString();
    var batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return batchScheduleImplementation.GenerateNewScheduleBatchExecution(executionCode, scheduleData.id);
    })
        .then(function (_) {
        return batchScheduleImplementation.GetNewScheduledBatchExecutions(executionCode, scheduleData.id);
    })
        .then(function (_) {
        logger.info('[BatchScheduleService::CreateBatchScheduleExecutionsData] Finishing function');
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[BatchScheduleService::CreateBatchScheduleExecutionsData] Function failed');
        return Promise.reject(err);
    });
}
function DeleteBatchsScheduleExecutionsData(vcc, scheduleData) {
    var executionCode = moment().toDate().toString();
    var batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return batchScheduleImplementation.RemoveScheduleBatchExecutions(vcc, scheduleData, executionCode);
    })
        .then(function (_) {
    })
        .then(function (_) {
        logger.info('[BatchScheduleService::DeleteBatchsScheduleExecutionsData] Finishing function');
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[BatchScheduleService::DeleteBatchsScheduleExecutionsData] Function failed');
        return Promise.reject(err);
    });
}
function GetBatchScheduledTemplateData(data, latestResults) {
    var firstLevel = [];
    var uniqueCategory1Objects = arrays_shared_1.GetUniqueFromObjectArray(data.filter(function (r) { return r.category1; }), 'category1');
    var nullCategory1Objects = arrays_shared_1.GetUniqueFromObjectArray(data.filter(function (r) { return !r.category1; }), 'description');
    uniqueCategory1Objects.map(function (uc1o) {
        var currentSecondLevel = [];
        var uniqueCategory2Objects = arrays_shared_1.GetUniqueFromObjectArray(data.filter(function (r) { return r.category1 === uc1o.category1 && r.category2 !== null; }), 'category2');
        var nullCategory2Objects = arrays_shared_1.GetUniqueFromObjectArray(data.filter(function (r) { return r.category1 === uc1o.category1 && r.category2 === null; }), 'description');
        uniqueCategory2Objects.map(function (uc2o) {
            var currentThirdLevel = [];
            var uniqueDescriptionObjects = arrays_shared_1.GetUniqueFromObjectArray(data.filter(function (r) { return r.category1 === uc1o.category1 && r.category2 === uc2o.category2; }), 'description');
            uniqueDescriptionObjects.map(function (udo) {
                var itemL3 = {
                    isFinalLevel: true,
                    name: udo.description,
                    batchScheduledTemplate: udo.batchScheduleId,
                    colname: udo.batchScheduleId + "-L4",
                    level: 4,
                    value: data.filter(function (r) { return r.category1 === uc2o.category1 && r.category2 === uc2o.category2 && r.description === udo.description; }).length,
                    total: data.filter(function (r) { return r.category1 === uc2o.category1 && r.category2 === uc2o.category2 && r.description === udo.description; }).length,
                    category1: udo.category1,
                    category2: udo.category2,
                    description: udo.description
                };
                currentThirdLevel.push(itemL3);
            });
            var itemL2 = {
                isFinalLevel: false,
                name: uc2o.category2,
                batchScheduledTemplate: uc2o.batchScheduleId,
                colname: uc2o.batchScheduleId + "-L3",
                level: 3,
                items: currentThirdLevel,
                value: arrays_shared_1.GetSumFromObjectArray(currentThirdLevel, 'total'),
                category1: uc2o.category1,
                category2: null,
                description: uc2o.category2
            };
            itemL2.total = itemL2.value;
            currentSecondLevel.push(itemL2);
        });
        nullCategory2Objects.map(function (nc2o) {
            var itemL2 = {
                isFinalLevel: true,
                name: nc2o.description,
                batchScheduledTemplate: nc2o.batchScheduleId,
                colname: nc2o.batchScheduleId + "-L3",
                level: 3,
                value: data.filter(function (r) { return r.category1 === nc2o.category1 && r.category2 === null && r.description === nc2o.description; }).length,
                total: data.filter(function (r) { return r.category1 === nc2o.category1 && r.category2 === null && r.description === nc2o.description; }).length,
                category1: nc2o.category1,
                category2: null,
                description: nc2o.description
            };
            currentSecondLevel.push(itemL2);
        });
        var itemL1 = {
            isFinalLevel: false,
            name: uc1o.category1,
            batchScheduledTemplate: uc1o.batchScheduleId,
            colname: uc1o.batchScheduleId + "-L2",
            level: 2,
            items: currentSecondLevel,
            value: arrays_shared_1.GetSumFromObjectArray(currentSecondLevel, 'total'),
            category1: null,
            category2: null,
            description: uc1o.category1
        };
        itemL1.total = itemL1.value;
        if (currentSecondLevel.length === 0) {
            itemL1.isFinalLevel = true;
            itemL1.value = data.filter(function (r) { return r.category1 === uc1o.category1; }).length;
            itemL1.total = data.filter(function (r) { return r.category1 === uc1o.category1; }).length;
        }
        firstLevel.push(itemL1);
    });
    nullCategory1Objects.map(function (nc1o) {
        var itemL1 = {
            isFinalLevel: true,
            name: nc1o.description,
            batchScheduledTemplate: nc1o.batchScheduleId,
            colname: nc1o.batchScheduleId + "-L2",
            level: 2,
            value: data.filter(function (r) { return r.category1 === null && r.description === nc1o.description; }).length,
            total: data.filter(function (r) { return r.category1 === null && r.description === nc1o.description; }).length,
            category1: null,
            category2: null,
            description: nc1o.description
        };
        firstLevel.push(itemL1);
    });
    return firstLevel;
}
function SaveProcessBatchSchedule(vcc, vccType, process, userId) {
    var res = false;
    process.scheduleData.firstExecutionDate = moment(process.scheduleData.firstExecutionDate).utc().toDate();
    process.scheduleData.startDate = moment(process.scheduleData.startDate).utc().toDate();
    process.scheduleData.endDate = process.scheduleData.endDate ? moment(process.scheduleData.endDate).utc().toDate() : null;
    var includeCallChannel = process.processFilters.filter(function (pf) { return pf.key === 'InteractionType' && pf.value === 'CALL'; }).length ? true : false;
    var includeTextChannel = process.processFilters.filter(function (pf) { return pf.key === 'InteractionType' && pf.value !== 'CALL'; }).length ? true : false;
    var executionDelay = 0;
    if (vccType === 'I6' && includeCallChannel)
        executionDelay = 1;
    if (includeTextChannel)
        executionDelay = 24;
    process.scheduleData.batchData = {
        clientId: process.clientId,
        description: process.description,
        type: process.type,
        language: process.language,
        dictionaryId: process.dictionaryId,
        environment: process.environment,
        environmentType: process.environmentType,
        executionDelay: executionDelay,
        samplePercentage: process.percentage,
        vccType: vccType
    };
    process.scheduleData.executionDelay = executionDelay;
    var executionList = [];
    process.scheduleData.executionList.map(function (el, i) {
        var execution = new schedule_model_1.ScheduleExecution();
        execution.batchScheduleId = process.scheduleData.id;
        execution.order = i + 1,
            execution.id = utilities_1.guid();
        execution.batchId = null;
        execution.executionDate = moment(el).toDate();
        execution.status = 'PENDING';
        execution.createdByUserId = userId;
        executionList.push(execution);
    });
    process.scheduleData.executionList = executionList;
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return SaveBatchSchedule(vcc, process.scheduleData, userId);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_BATCH_SCHEDULE_SAVE_FAILED';
        var promiseArray = [];
        process.processFilters.map(function (pf) {
            promiseArray.push(mssql_service_1.MSSql.SaveProcessFilter(vcc, pf.processId, pf.key, pf.value, moment().utc().toDate(), pf.createdByUserId, moment().utc().toDate(), pf.lastModifiedByUserId));
        });
        return Promise.all(promiseArray);
    })
        .then(function (results) {
        logger.info('[BatchScheduleService::SaveProcessBatchSchedule] Finishing function');
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return Promise.resolve(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .catch(function (err) {
        logger.error('[BatchScheduleService::SaveProcessBatchSchedule] Function failed');
        return Promise.reject(err);
    });
}
exports.SaveProcessBatchSchedule = SaveProcessBatchSchedule;
function SaveTemplateBatchSchedule(vcc, template, userId) {
    template.scheduleData.id = template.name;
    template.scheduleData.batchData = template.template;
    template.scheduleData.processFiltersId = utilities_1.guid();
    template.scheduleData.startDate = moment(template.scheduleData.startDate).utc().toDate();
    template.scheduleData.firstExecutionDate = moment(template.scheduleData.firstExecutionDate).utc().toDate();
    template.scheduleData.startDate = moment(template.scheduleData.firstExecutionDate).utc().toDate();
    if (template.scheduleData.endDate)
        template.scheduleData.endDate = moment(template.scheduleData.endDate).utc().toDate();
    template.scheduleData.daysInterval = null;
    template.scheduleData.executionHour = null;
    template.scheduleData.hoursInterval = null;
    template.scheduleData.monthDay = null;
    template.scheduleData.weekDay = null;
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return SaveBatchSchedule(vcc, template.scheduleData, userId);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_BATCH_SCHEDULE_SAVE_FAILED';
        if (template.scheduleData.includeHistoricalData) {
        }
        template.name = template.scheduleData.processFiltersId;
        return template_service_1.CreateTemplate(vcc, template, userId);
    })
        .then(function (_) {
        logger.info('[BatchScheduleService::SaveTemplateBatchSchedule] Finishing function');
        return Promise.resolve(null);
    })
        .catch(function (err) {
        logger.error('[BatchScheduleService::SaveTemplateBatchSchedule] Function failed');
        return Promise.reject(err);
    });
}
exports.SaveTemplateBatchSchedule = SaveTemplateBatchSchedule;
function SaveEvaluationPlanBatchSchedule() {
    return promises_shared_1.DummyPromise()
        .then(function (_) {
    })
        .then(function (_) {
        logger.info('[BatchScheduleService::SaveEvaluationPlanBatchSchedule] Finishing function');
        return Promise.resolve(null);
    })
        .catch(function (err) {
        logger.error('[BatchScheduleService::SaveEvaluationPlanBatchSchedule] Function failed');
        return Promise.reject(err);
    });
}
exports.SaveEvaluationPlanBatchSchedule = SaveEvaluationPlanBatchSchedule;
function GenerateNewBatcScheduleExecutions(schedule, range, lastExecutionOrder) {
    if (!range)
        range = 7;
    if (!lastExecutionOrder)
        lastExecutionOrder = 0;
    var lastExecutionDate = moment(schedule.lastExecutionDate);
    var maxLastExecutionDate = moment(schedule.endDate ? schedule.endDate : moment().add(range, 'days'));
    var executionList = [];
    while (lastExecutionDate.isSameOrBefore(maxLastExecutionDate)) {
        switch (schedule.frequency) {
            case 'daily':
                lastExecutionDate.add(1, 'days');
                break;
            case 'weekly':
                lastExecutionDate.add(1, 'weeks');
                break;
            case 'monthly':
                lastExecutionDate.add(1, 'months');
                break;
            case 'custom_per_days':
                lastExecutionDate.add(schedule.daysInterval, 'days');
                break;
            case 'custom_per_hours':
                lastExecutionDate.add(schedule.hoursInterval, 'hours');
                break;
            default: break;
        }
        var execution = new schedule_model_1.ScheduleExecution();
        execution.vcc = schedule.vcc;
        execution.batchScheduleId = schedule.id;
        execution.order = ++lastExecutionOrder;
        execution.id = utilities_1.guid();
        execution.batchId = null;
        execution.executionDate = lastExecutionDate.toDate();
        execution.status = 'PENDING';
        execution.createdByUserId = schedule.createdByUserId;
        console.log('RG-LOG: fecha de ejecución', lastExecutionDate.toDate());
        if (lastExecutionDate.isSameOrAfter(moment()) && lastExecutionDate.isBefore(maxLastExecutionDate))
            executionList.push(execution);
    }
    console.log(executionList);
    return executionList;
}
exports.GenerateNewBatcScheduleExecutions = GenerateNewBatcScheduleExecutions;
