"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SubgroupService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var logger = log4js.getLogger('ServerScripts');
function SubgroupService(app) {
    app.get('/api/subgroup/get_subgroups/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetSubgroups(instance, search_criteria, search_expression, disabled);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var subgroupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: subgroupMsg, error: errorMsg });
        });
    });
    app.post('/api/subgroup/create_subgroup', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newSubgroup = (req.body || {});
        newSubgroup.createdDate = nowUtc;
        newSubgroup.createdByUserId = userId;
        newSubgroup.lastModifiedDate = nowUtc;
        newSubgroup.lastModifiedByUserId = userId;
        newSubgroup.description = newSubgroup.description.trim();
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetSubgroupDescription(instance, newSubgroup.description);
        }).then(function (result) {
            if (result)
                throw 'SUBGROUP_SERVER_ERROR_EXISTS';
            newSubgroup.id = utilities_1.guid();
            return SaveSubgroup(instance, newSubgroup);
        }).then(function (result) {
            if (!result)
                throw 'SUBGROUP_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'SUBGROUP_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/subgroup/get_subgroup/:subgroup_id', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var subgroupId = req.params.subgroup_id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!subgroupId)
                throw 'SUBGROUP_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetSubgroup(instance, subgroupId);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/subgroup/save_subgroup', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newSubgroup = (req.body || {});
        newSubgroup.createdDate = null;
        newSubgroup.createdByUserId = null;
        newSubgroup.lastModifiedDate = nowUtc;
        newSubgroup.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetSubgroupDescription(instance, newSubgroup.description);
        }).then(function (result) {
            var subGrupDesc = {};
            subGrupDesc = result || subGrupDesc;
            if (result && subGrupDesc.id !== newSubgroup.id)
                throw 'SUBGROUP_SERVER_ERROR_EXISTS';
            return SaveSubgroup(instance, newSubgroup);
        }).then(function (resultSave) {
            if (!resultSave)
                throw 'SUBGROUP_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(true);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'SUBGROUP_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/subgroup/delete_subgroup', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var subgroup = (req.body || {});
        var subgroupId = subgroup.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!subgroup)
                throw 'SUBGROUP_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteSubgroup(instance, subgroupId);
        }).then(function (resultDelete) {
            if (!resultDelete)
                throw 'SUBGROUP_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        }).then(function (resultDelete) {
            if (res.headersSent)
                return;
            res.json({ status: resultDelete, description: (resultDelete ? 'SERVER_SUCCESS_MESSAGE' : 'SUBGROUP_SERVER_ERROR_ON_DELETE'), data: resultDelete });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/subgroup/delete_subgroups', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var subgroups = req.body.subgroups || [];
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!subgroups)
                throw 'SUBGROUP_SERVER_ERROR_MISSING_IDS';
            var promiseArray = [];
            subgroups.forEach(function (subgroup) {
                promiseArray.push(mssql_service_1.MSSql.DeleteSubgroup(instance, subgroup.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteSubgroups) {
            var result;
            if (resultDeleteSubgroups.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Groups eliminados correctamente');
            }
            return result;
        }).then(function (resultDelete) {
            if (res.headersSent)
                return;
            res.json({ status: resultDelete, description: (resultDelete ? 'SERVER_SUCCESS_MESSAGE' : 'SUBGROUP_SERVER_ERROR_ON_DELETE'), data: resultDelete });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.SubgroupService = SubgroupService;
function SaveSubgroup(instance, subgroup) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveSubgroup(instance, subgroup.id, subgroup.description, subgroup.createdDate, subgroup.createdByUserId, subgroup.lastModifiedDate, subgroup.lastModifiedByUserId, subgroup.enabled));
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
