"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TagService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function TagService(app) {
    app.get('/api/tag/get_tags/:audioId/:section/:part?', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var audioId = req.params.audioId || '';
        var section = Number(req.params.section);
        var part = Number(req.params.part);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetTags(vcc, audioId, section, part);
        }).then(function (resultRoles) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultRoles });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: "ERROR", datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/tag/get_unique_tags', function (req, res, next) {
        logger.info('[TagService] Received ' + req.method + ' method on  \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[TagService] Requesting all unique tags from MSSql.');
            return mssql_service_1.MSSql.GetUniqueTags(vcc);
        }).then(function (resultGetUniqueTags) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultGetUniqueTags });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error on request');
            logger.error('[TagService] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: "ERROR", datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/tag/save_tag', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newTag = (req.body || {});
        newTag.createdDate = nowUtc;
        newTag.createdByUserId = userId;
        newTag.lastModifiedDate = nowUtc;
        newTag.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!newTag.tag)
                throw 'ERROR_TAG_NAME_MISSING';
            return mssql_service_1.MSSql.SaveTag(vcc, newTag.audioId, newTag.section, newTag.part, newTag.tag, newTag.currentTime, newTag.comment, newTag.createdDate, newTag.createdByUserId, newTag.lastModifiedDate, newTag.lastModifiedByUserId);
        }).then(function (resultSaveTag) {
            if (res.headersSent)
                return;
            logger.info('Comentario \'' + newTag.id + '\' guardado correctamente');
            res.json({ status: resultSaveTag, description: (resultSaveTag ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: resultSaveTag });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: "ERROR", datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/tag/delete_tag', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var newTag = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.DeleteTag(vcc, newTag.audioId, newTag.section, newTag.part, newTag.tag);
        }).then(function (resultSaveTag) {
            if (res.headersSent)
                return;
            logger.info('Comentario \'' + newTag.id + '\' eliminado correctamente');
            res.json({ status: resultSaveTag, description: (resultSaveTag ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: resultSaveTag });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: "ERROR", datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.get('/api/tag/get_all_tags?', function (req, res, next) {
        logger.info('[TagService::get_all_tags] Received ' + req.method + ' method on  \'' + req.path + '\'');
        var instance = res.locals.instance;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[TagService::get_all_tags] Requesting all tags from MSSql SP GetAllTags');
            return mssql_service_1.MSSql.GetAllTags(instance);
        }).then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[TagService::get_all_tags] Received resultro from MSSql');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error on request');
            logger.error('[TagService::get_all_tags] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TagService = TagService;
