"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TipificationService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var arrays_shared_1 = require("../shared/arrays.shared");
var logger = log4js.getLogger('ServerScripts');
function TipificationService(app) {
    app.post('/api/tipification/create_tipification', function (req, res, next) {
        logger.info("[TipificationService::create_tipification] Received " + req.method + " method on '" + req.path + "'");
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var newTipification = (req.body || {});
        var wasCreated = false;
        newTipification.description = newTipification.description.trim();
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetTipifications(vcc, 'description', newTipification.description);
        })
            .then(function (result) {
            if (result === null || result === void 0 ? void 0 : result.length)
                throw 'SERVER_ERROR_TIPIFICATION_ALREADY_EXISTS';
            newTipification.id = newTipification.id ? newTipification.id : utilities_1.guid();
            return mssql_service_1.MSSql.SaveTipification(vcc, newTipification.id, newTipification.description, newTipification.detail, userId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_TIPIFICATION_SAVE_FAILED';
            wasCreated = true;
            return SaveTipificationItems(vcc, newTipification, userId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_TIPIFICATION_ITEMS_SAVE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error("[TipificationService::create_tipification] Error on '" + req.path + "': " + errorMsg);
            if (wasCreated)
                mssql_service_1.MSSql.DeleteTipification(vcc, newTipification.id);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/tipification/save_tipification', function (req, res, next) {
        logger.info("[TipificationService::save_tipification] Received " + req.method + " method on '" + req.path + "'");
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var tipification = (req.body || {});
        tipification.description = tipification.description.trim();
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.SaveTipification(vcc, tipification.id, tipification.description, tipification.detail, userId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_TIPIFICATION_SAVE_FAILED';
            return SaveTipificationItems(vcc, tipification, userId);
        })
            .then(function (resultSave) {
            if (!resultSave)
                throw 'SERVER_ERROR_TIPIFICATION_ITEMS_SAVE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: 'SERVER_SUCCESS_MESSAGE', data: resultSave });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error("[TipificationService::create_tipification] Error on '" + req.path + "': " + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/tipification/get_tipifications/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info("[TipificationService::get_tipifications] Received " + req.method + " method on '" + req.path + "'");
        var vcc = res.locals.vcc;
        var searchCriteria = req.params.search_criteria || '';
        var searchExpression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetTipifications(vcc, searchCriteria, searchExpression);
        })
            .then(function (tipifications) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: tipifications });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error("[TipificationService::get_tipifications] Error on '" + req.path + "': " + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/tipification/get_tipification/:tipificationId', function (req, res, next) {
        logger.info("[TipificationService::get_tipification] Received " + req.method + " method on '" + req.path + "'");
        var vcc = res.locals.vcc;
        var tipification = {};
        var tipificationId = req.params.tipificationId;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!tipificationId)
                throw 'SERVER_ERROR_MISSING_TIPIFICATION_ID';
            return mssql_service_1.MSSql.GetTipification(vcc, tipificationId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_TIPIFICATION_NOT_FOUND';
            tipification = result;
            return mssql_service_1.MSSql.GetTipificationItems(vcc, tipification.id);
        })
            .then(function (result) {
            tipification.items = result ? result : [];
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: tipification });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error("[TipificationService::get_tipification] Error on '" + req.path + "': " + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/tipification/delete_tipification', function (req, res, next) {
        logger.info("[TipificationService::delete_tipification] Received " + req.method + " method on '" + req.path + "'");
        var vcc = res.locals.vcc;
        var tipification = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!tipification.id)
                throw 'SERVER_ERROR_MISSING_TIPIFICATION_ID';
            return mssql_service_1.MSSql.DeleteTipification(vcc, tipification.id);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_TIPIFICATION_DELETE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: result, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error("[TipificationService::get_tipification] Error on '" + req.path + "': " + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/tipification/delete_tipifications', function (req, res, next) {
        logger.info("[TipificationService::delete_tipifications] Received " + req.method + " method on '" + req.path + "'");
        var vcc = res.locals.vcc;
        var tipifications = req.body.tipifications || [];
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!tipifications)
                throw 'SERVER_ERROR_MISING_TIPIFICATIONS';
            var promiseArray = [];
            tipifications.forEach(function (t) { return promiseArray.push(mssql_service_1.MSSql.DeleteTipification(vcc, t.id)); });
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            var result;
            if (results.filter(function (r) { return r !== true; }).length) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Vccs eliminados correctamente');
            }
            return result;
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error("[TipificationService::get_tipification] Error on '" + req.path + "': " + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TipificationService = TipificationService;
function SaveTipificationItems(vcc, tipification, userId) {
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return mssql_service_1.MSSql.GetTipificationItems(vcc, tipification.id);
    })
        .then(function (result) {
        var _a, _b, _c, _d;
        var tipificationItems = result ? result : [];
        var classifiedItems = arrays_shared_1.GetCompareItemsFromObjectArray(tipificationItems, tipification.items, 'id');
        var promiseArray = [];
        if ((_b = (_a = tipification.tipificationQuery) === null || _a === void 0 ? void 0 : _a.queryId) === null || _b === void 0 ? void 0 : _b.length) {
            promiseArray.push(mssql_service_1.MSSql.SaveTipificationQuery(vcc, tipification.id, tipification.tipificationQuery.queryId, tipification.tipificationQuery.queryContent, tipification.tipificationQuery.acceptancePercentage, userId));
        }
        else
            promiseArray.push(mssql_service_1.MSSql.DeleteTipificationQuery(vcc, tipification.id));
        if ((_c = tipification === null || tipification === void 0 ? void 0 : tipification.items) === null || _c === void 0 ? void 0 : _c.length) {
            tipification.items.map(function (i) {
                i.tipificationId = tipification.id;
                promiseArray.push(mssql_service_1.MSSql.SaveTipificationItem(vcc, tipification.id, i.id, i.secondLevelItemId, i.description, i.order, userId));
            });
        }
        if ((_d = classifiedItems === null || classifiedItems === void 0 ? void 0 : classifiedItems.removed) === null || _d === void 0 ? void 0 : _d.length)
            classifiedItems.removed.forEach(function (i) { return promiseArray.push(mssql_service_1.MSSql.DeleteTipificationItem(vcc, i.id)); });
        if (!promiseArray.length)
            promiseArray.push(Promise.resolve(true));
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        var saveResult = false;
        if (result.filter(function (r) { return (r !== true); }).length)
            saveResult = false;
        else
            saveResult = true;
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error("[TipificationService::get_tipifications] Error: " + errorMsg);
        return Promise.resolve(false);
    });
}
