"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SimpleDeleteRequest = exports.SimplePutRequest = exports.SimplePostRequest = exports.SimpleGetRequest = exports.AttachedFilePostRequest = void 0;
var log4js = require("log4js");
var axios_1 = require("axios");
var http = require('http');
var https = require('https');
var logger = log4js.getLogger('ServerScripts');
function SimpleResult(rawResponse) {
    var requestResponse = null;
    if (typeof rawResponse === 'string') {
        try {
            requestResponse = JSON.parse(rawResponse);
        }
        catch (err) {
            requestResponse = null;
        }
    }
    else {
        requestResponse = rawResponse;
    }
    return requestResponse;
}
function AttachedFilePostRequest(data, formData, formDataBackup) {
    var requestConfig = {
        method: 'POST',
        timeout: 300000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: {
            'Content-Type': "multipart/form-data; boundary=" + formData._boundary,
            'Connection': 'keep-alive'
        },
        maxContentLength: Infinity,
        maxBodyLength: Infinity,
        data: formData
    };
    return axios_1.default(requestConfig)
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error("[request.shared::AttachedFilePostRequest] Getting first try request error: " + errorMessage + " from \"" + requestConfig.baseURL + "\"");
        if ((requestConfig.baseURL.includes('http:')
            ||
                requestConfig.baseURL.includes('https:'))
            &&
                data.retryProtocol) {
            if (requestConfig.baseURL.includes('http:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('http:', 'https:');
            else if (requestConfig.baseURL.includes('https:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('https', 'http');
            logger.warn("[request.shared::AttachedFilePostRequest] Failed \"" + data.uri + "\" retrying \"" + requestConfig.baseURL + "\"");
            return axios_1.default(requestConfig);
        }
        else if (data.retryUri) {
            logger.warn("[request.shared::AttachedFilePostRequest] Failed \"" + data.uri + "\", retrying \"" + data.retryUri + "\"");
            requestConfig = {
                method: 'POST',
                timeout: 300000,
                proxy: false,
                httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
                httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
                baseURL: data.retryUri,
                headers: {
                    'Content-Type': "multipart/form-data; boundary=" + formDataBackup._boundary,
                    'Connection': 'keep-alive'
                },
                maxContentLength: Infinity,
                maxBodyLength: Infinity,
                data: formDataBackup
            };
            return axios_1.default(requestConfig);
        }
        else
            throw err;
    })
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return requestResponse.data;
        }
        else {
            throw 'SERVER_ERROR_ATTACHED_FILE_POST_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::AttachedFileRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.AttachedFilePostRequest = AttachedFilePostRequest;
function SimpleGetRequest(data) {
    var requestConfig = {
        method: 'GET',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers
    };
    if (data.responseType)
        requestConfig.responseType = data.responseType;
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return axios_1.default(requestConfig)
        .catch(function (err) {
        var _a;
        if ((data === null || data === void 0 ? void 0 : data.validateErrorResponse) && ((_a = err === null || err === void 0 ? void 0 : err.response) === null || _a === void 0 ? void 0 : _a.data))
            err = JSON.stringify(err.response.data);
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description || JSON.stringify(err));
        logger.error('[request.shared::SimpleGetRequest] Getting first try request error:', err);
        logger.error("[request.shared::SimpleGetRequest] Getting first try request error: " + errorMessage + " from \"" + requestConfig.baseURL + "\"");
        if ((requestConfig.baseURL.includes('http:')
            ||
                requestConfig.baseURL.includes('https:'))
            &&
                data.retryProtocol) {
            if (requestConfig.baseURL.includes('http:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('http:', 'https:');
            else if (requestConfig.baseURL.includes('https:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('https', 'http');
            logger.warn("[request.shared::SimpleGetRequest] Failed \"" + data.uri + "\" retrying \"" + requestConfig.baseURL + "\"");
            return axios_1.default(requestConfig);
        }
        else if (data.retryUri) {
            logger.warn("[request.shared::SimpleGetRequest] Failed \"" + data.uri + "\", retrying \"" + data.retryUri + "\"");
            requestConfig.baseURL = data.retryUri;
            return axios_1.default(requestConfig);
        }
        else
            throw err;
    })
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_GET_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var _a;
        if ((data === null || data === void 0 ? void 0 : data.validateErrorResponse) && ((_a = err === null || err === void 0 ? void 0 : err.response) === null || _a === void 0 ? void 0 : _a.data))
            err = JSON.stringify(err.response.data);
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description || JSON.stringify(err));
        logger.error('[request.shared::SimpleGetRequest] Getting second error:', errorMessage);
        if (data === null || data === void 0 ? void 0 : data.uri)
            logger.error('[request.shared::SimpleGetRequest] Failed for:', data.uri);
        return Promise.reject(err);
    });
}
exports.SimpleGetRequest = SimpleGetRequest;
function SimplePostRequest(data) {
    var requestConfig = {
        method: 'POST',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    if (data.responseType)
        requestConfig.responseType = data.responseType;
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return axios_1.default(requestConfig)
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error("[request.shared::SimplePostRequest] Getting first try request error: " + errorMessage + " from \"" + requestConfig.baseURL + "\"");
        if ((requestConfig.baseURL.includes('http:')
            ||
                requestConfig.baseURL.includes('https:'))
            &&
                data.retryProtocol) {
            if (requestConfig.baseURL.includes('http:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('http:', 'https:');
            else if (requestConfig.baseURL.includes('https:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('https', 'http');
            logger.warn("[request.shared::SimplePostRequest] Failed \"" + data.uri + "\" retrying \"" + requestConfig.baseURL + "\"");
            return axios_1.default(requestConfig);
        }
        else if (data.retryUri) {
            logger.warn("[request.shared::SimplePostRequest] Failed \"" + data.uri + "\", retrying \"" + data.retryUri + "\"");
            requestConfig.baseURL = data.retryUri;
            return axios_1.default(requestConfig);
        }
        else
            throw err;
    })
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_POST_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimplePostRequest] Getting second error:', errorMessage);
        if (data === null || data === void 0 ? void 0 : data.uri)
            logger.error('[request.shared::SimpleGetRequest] Failed for:', data.uri);
        return Promise.reject(err);
    });
}
exports.SimplePostRequest = SimplePostRequest;
function SimplePutRequest(data) {
    var requestConfig = {
        method: 'PUT',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return axios_1.default(requestConfig)
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_DELETE_PUT_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimplePutRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.SimplePutRequest = SimplePutRequest;
function SimpleDeleteRequest(data) {
    var requestConfig = {
        method: 'DELETE',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return axios_1.default(requestConfig)
        .then(function (result) {
        var requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_DELETE_REQUEST_FAILED';
        }
    })
        .catch(function (err) {
        var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimpleDeleteRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.SimpleDeleteRequest = SimpleDeleteRequest;
