"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InitTask = void 0;
var express = require("express");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var mssql_config_1 = require("../config/mssql.config");
var mssql_init_1 = require("../db/mssql.init");
var promises_shared_1 = require("../shared/promises.shared");
var redis_init_1 = require("../db/redis.init");
var process_implementation_1 = require("../implementation/process.implementation");
var audio_transfer_implementation_1 = require("../implementation/audio-transfer.implementation");
var logger_config_1 = require("../config/logger.config");
var transcriptions_implementation_1 = require("../implementation/transcriptions.implementation");
var semantic_analysis_implementation_1 = require("../implementation/semantic-analysis.implementation");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var redis_service_1 = require("../db/redis.service");
var moment = require("moment");
var supervisor_implementation_1 = require("../implementation/supervisor.implementation");
var cleaner_implementation_1 = require("../implementation/cleaner.implementation");
var file_manager_1 = require("../shared/file-manager");
var semantic_query_implementation_1 = require("../implementation/semantic-query.implementation");
var config_1 = require("../config/config");
var billing_batch_status_implementation_1 = require("../implementation/billing-batch-status.implementation");
var diarization_convert_implementation_1 = require("../implementation/diarization-convert.implementation");
var diarization_analysis_implementation_1 = require("../implementation/diarization-analysis.implementation");
var train_service_1 = require("../services/train.service");
var utterance_service_1 = require("../services/utterance.service");
var cognitive_engine_migrate_implementation_1 = require("../implementation/cognitive-engine-migrate.implementation");
var nlu_service_1 = require("../services/nlu.service");
var arrays_shared_1 = require("../shared/arrays.shared");
var i6_update_implementation_1 = require("../implementation/i6-update.implementation");
var batch_schedule_implementation_1 = require("../implementation/batch-schedule.implementation");
var tipification_analysis_implementation_1 = require("../implementation/tipification-analysis.implementation");
var sentiment_analysis_implementation_1 = require("../implementation/sentiment-analysis.implementation");
log4js.configure(logger_config_1.LOGGER_CONFIG);
var logger = log4js.getLogger('Task');
function InitTask() {
    mssql_config_1.MSSQL_CONFIG.connectionTimeout = 300 * 1000;
    mssql_config_1.MSSQL_CONFIG.requestTimeout = 300 * 1000;
    var sqlConn;
    var redisClient;
    var executionCode = Date.now().toString();
    var app = express();
    logger.info('[schedule.task.ts::InitTask] begin Task executionCode:', executionCode);
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return redis_init_1.RedisInit(app);
    })
        .then(function (_) {
        redisClient = app.get('redisClient');
        return mssql_init_1.MsSqlInit(app);
    })
        .then(function (_) {
        sqlConn = app.get('sqlConn');
        var supervisor = new supervisor_implementation_1.SupervisorImplementation();
        return supervisor.Execute(executionCode);
    })
        .then(function (_) {
        return InitCognitiveModel();
    })
        .then(function (_) {
        return StopAllRunningRasaModelFromTest();
    })
        .then(function (_) {
        return TrainSystemCognitiveModel();
    })
        .then(function (_) {
        return InitBatchSchedule(executionCode);
    })
        .then(function (result) {
        if (!result)
            logger.warn('[schedule.task::InitTask] InitCognitiveModel function failed');
        var billingBatchStatus = new billing_batch_status_implementation_1.BillingBatchStatusImplementation();
        return billingBatchStatus.Execute(executionCode);
    })
        .then(function (result) {
        var _a, _b;
        if (!(result && result.data && result.data['allowedBatches'])
            &&
                !(result && result.data && result.data['cancelledBatches']))
            logger.debug('[schedule.task.ts::InitTask] Result from billingBatchStatus: SERVER_ERROR_BATCHES_NOT_FOUND');
        var promiseArray = [];
        var process = new process_implementation_1.ProcessImplementation();
        var i6Update = new i6_update_implementation_1.I6UpdateImplementation();
        var audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
        var diarizarionConvert = new diarization_convert_implementation_1.DiarizationConvertImplementation();
        var transcriptions = new transcriptions_implementation_1.TranscriptionsImplementation();
        var diarizarionAnalysis = new diarization_analysis_implementation_1.DiarizationAnalysisImplementation();
        var semanticAnalysis = new semantic_analysis_implementation_1.SemanticAnalysisImplementation();
        var semanticQuery = new semantic_query_implementation_1.SemanticQueryImplementation();
        var cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
        var cognitiveEngineMigationImplementation = new cognitive_engine_migrate_implementation_1.CognitiveEngineMigrateImplementation();
        var batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
        var tipificationAnalysisImplementation = new tipification_analysis_implementation_1.TipificationAnalysisImplementation();
        var sentimentAnalysis = new sentiment_analysis_implementation_1.SentimentAnalysisImplementation();
        promiseArray.push(process.Execute(executionCode, config_1.RATE_RECURSIVE_PROCESS), i6Update.Execute(executionCode, config_1.RATE_RECURSIVE_I6_UPDATE), audioTransfer.Execute(executionCode, config_1.RATE_RECURSIVE_AUDIO_TRANSFER), diarizarionConvert.Execute(executionCode, config_1.RATE_RECURSIVE_DIARIZATION_CONVERT), transcriptions.Execute(executionCode, config_1.RATE_RECURSIVE_TRANSCRIPTION, result.data && result.data.allowedBatches.transcription ? result.data.allowedBatches.transcription : [], result.data && result.data.cancelledBatches.transcription ? result.data.cancelledBatches.transcription : []), diarizarionAnalysis.Execute(executionCode, config_1.RATE_RECURSIVE_DIARIZATION_ANALYSIS), sentimentAnalysis.Execute(executionCode, config_1.RATE_RECURSIVE_SENTIMENT_ANALYSIS), semanticAnalysis.Execute(executionCode, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS), semanticQuery.Execute(executionCode, config_1.RATE_RECURSIVE_SEMANTIC_QUERY, result.data && result.data.allowedBatches.semanticAnalysis ? result.data.allowedBatches.semanticAnalysis : [], result.data && result.data.cancelledBatches.semanticAnalysis ? result.data.cancelledBatches.semanticAnalysis : []), cognitiveAnalysis.Execute(executionCode, config_1.RATE_RECURSIVE_COGNITIVE_ANALYSIS, result.data && result.data.allowedBatches.cognitiveAnalysis ? result.data.allowedBatches.cognitiveAnalysis : [], result.data && result.data.cancelledBatches.cognitiveAnalysis ? result.data.cancelledBatches.cognitiveAnalysis : []), cognitiveEngineMigationImplementation.Execute(executionCode), batchScheduleImplementation.Execute(executionCode, config_1.RATE_RECURSIVE_BATCH_SCHEDULE), tipificationAnalysisImplementation.Execute(executionCode, config_1.RATE_RECURSIVE_TIPIFICATION_ANALYSIS, ((_a = result.data) === null || _a === void 0 ? void 0 : _a.allowedBatches.cognitiveAnalysis) ? result.data.allowedBatches.cognitiveAnalysis : [], ((_b = result.data) === null || _b === void 0 ? void 0 : _b.cancelledBatches.cognitiveAnalysis) ? result.data.cancelledBatches.cognitiveAnalysis : []));
        return Promise.all(promiseArray);
    })
        .then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            logger.error('[schedule.task::InitTask] executionCode:', executionCode, 'getting error results');
            return Promise.resolve(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .then(function (_) {
        var promiseArray = [];
        promiseArray.push(file_manager_1.CleanupTempFile(config_1.COGNITIVE_ANALYSIS_FOLDER, config_1.PERSISTENCE));
        promiseArray.push(file_manager_1.CleanupTempFile(config_1.SEMANCTIC_ANALYSIS_FOLDER, config_1.PERSISTENCE));
        promiseArray.push(file_manager_1.CleanupTempFile(config_1.QUERY_FOLDER, config_1.PERSISTENCE));
        return Promise.all(promiseArray);
    })
        .then(function (_) {
        var cleaner = new cleaner_implementation_1.CleanerImplementation();
        return cleaner.Execute(executionCode);
    })
        .then(function (_) {
        logger.info('[schedule.task::InitTask] executionCode:', executionCode, 'Task has been finished');
        if (sqlConn)
            sqlConn.close();
        if (redisClient)
            redisClient.quit();
        setTimeout(function () {
            process.exit(0);
        }, 200);
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[schedule.task::InitTask] executionCode:', executionCode, 'Task has been finished', executionCode, 'getting error:', err);
        sqlConn.close();
        if (redisClient)
            redisClient.quit();
        setTimeout(function () {
            process.exit(0);
        }, 200);
        return Promise.resolve(false);
    });
}
exports.InitTask = InitTask;
function TrainSystemCognitiveModel() {
    var cognitiveService;
    var needUpdateNLUToUtterancesBulk = false;
    var needUpdateNLUToTrainDueToRasaUpdate = false;
    return promises_shared_1.DummyPromise().then(function (result) {
        return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
    })
        .then(function (result) {
        cognitiveService = result;
        return mssql_service_1.MSSql.GetNLUs('ALL', null, null, null, true);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_SYSTEM_APPS_NOT_FOUND';
        var promiseArray = [];
        result = arrays_shared_1.SortObjectArrayByShuffle(result);
        var timeOut = 0;
        result.forEach(function (r, i) {
            if (r.vcc !== 'system') {
                if (r.needTrain
                    &&
                        r.lastModifiedDate
                    &&
                        (moment().utc() > moment(r.lastModifiedDate).utc().add(5, 'minutes'))) {
                    logger.info('[schedule.task::TrainSystemCognitiveModel] Training system app:', r.appId);
                    timeOut = i * 1000;
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            return resolve(train_service_1.TrainApp(cognitiveService, null, r));
                        }, timeOut);
                    }));
                }
            }
        });
        result.filter(function (r) { return r.lastModifiedByUserId === 'dev-1'
            &&
                r.isSystemApp === true
            &&
                r.lastTrainedDateTime; })
            .map(function (n, i) {
            if (i === 0) {
                promiseArray.push(utterance_service_1.GetSaveUtterances(n));
            }
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        return nlu_service_1.GetNLUToUtterancesBulk(false);
    })
        .then(function (result) {
        var promiseArray = [];
        if (result && result.length) {
            needUpdateNLUToUtterancesBulk = true;
            result = arrays_shared_1.SortObjectArrayByShuffle(result);
            result.map(function (n, i) {
                if (n.lastModifiedByUserId === 'dev-1' && i === 0) {
                    promiseArray.push(utterance_service_1.GetSaveUtterances(n));
                }
            });
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        return nlu_service_1.GetNLUToTrainDueToRasaUpdate(false);
    })
        .then(function (result) {
        var promiseArray = [];
        if (result && result.length) {
            needUpdateNLUToTrainDueToRasaUpdate = true;
            result = arrays_shared_1.SortObjectArrayByShuffle(result);
            result.map(function (n, i) {
                if (i === 0
                    &&
                        n.needTrain
                    &&
                        n.lastModifiedByUserId === 'dev-2') {
                    promiseArray.push(train_service_1.TrainApp(cognitiveService, null, n));
                }
            });
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        var promiseArray = [];
        if (needUpdateNLUToUtterancesBulk)
            promiseArray.push(nlu_service_1.GetNLUToUtterancesBulk(true));
        if (needUpdateNLUToTrainDueToRasaUpdate)
            promiseArray.push(nlu_service_1.GetNLUToTrainDueToRasaUpdate(true));
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::TrainSystemCognitiveModel] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function InitCognitiveModel() {
    var cognitiveService;
    return promises_shared_1.DummyPromise().then(function (result) {
        return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
    }).then(function (result) {
        cognitiveService = result;
        var promiseArray = [];
        promiseArray.push(redis_service_1.Redis.GetCurrentBatchesRasaModels());
        promiseArray.push(redis_service_1.Redis.GetRunningRasaModels());
        return Promise.all(promiseArray);
    }).then(function (result) {
        logger.debug('[schedule.task::InitCognitiveModel] uniqueCurrentBatchesNlus: ', result[0]);
        logger.debug('[schedule.task::InitCognitiveModel] allRunningRasaModels: ', result[1]);
        var allCurrentBatchesNlus = [];
        if (result[0]) {
            var attributesArray = Object.getOwnPropertyNames(result[0]);
            attributesArray.map(function (aa) {
                allCurrentBatchesNlus = allCurrentBatchesNlus.concat(JSON.parse(result[0][aa].toLowerCase()));
            });
        }
        var uniqueCurrentBatchesNlus = allCurrentBatchesNlus.filter(function (item, i, ar) { return ar.indexOf(item) === i; });
        logger.debug('[schedule.task::InitCognitiveModel] uniqueCurrentBatchesNlus: ', uniqueCurrentBatchesNlus);
        var allRunningRasaModels = [];
        if (result[1]) {
            var attributesArray = Object.getOwnPropertyNames(result[1]);
            attributesArray.map(function (aa) {
                var arr = aa.split('::');
                allRunningRasaModels = allRunningRasaModels.concat(arr[1] + '::' + arr[2]);
            });
        }
        var uniqueRunningRasaModels = allRunningRasaModels.filter(function (item, i, ar) { return ar.indexOf(item) === i; });
        logger.debug('[schedule.task::InitCognitiveModel] uniqueRunningRasaModels: ', uniqueRunningRasaModels);
        var promiseArray = [];
        var index = 0;
        uniqueCurrentBatchesNlus.map(function (ucbn) {
            if (!uniqueRunningRasaModels.includes(ucbn)) {
                var timeOut_1 = index * 1000;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        return resolve(cognitiveService.PublishApp(null, null, ucbn, true));
                    }, timeOut_1);
                }));
                index++;
            }
        });
        uniqueRunningRasaModels.map(function (urrm) {
            if (!uniqueCurrentBatchesNlus.includes(urrm)) {
                var timeOut_2 = index * 1000;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        return resolve(cognitiveService.PublishApp(null, null, urrm, false));
                    }, timeOut_2);
                }));
                index++;
            }
        });
        return Promise.all(promiseArray);
    }).then(function (result) {
        logger.info('[schedule.task::InitCognitiveModel] Result: ' + result);
        return Promise.resolve(true);
    }).catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::InitCognitiveModel] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function StopAllRunningRasaModelFromTest() {
    var cognitiveService;
    return promises_shared_1.DummyPromise().then(function (result) {
        return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
    }).then(function (result) {
        cognitiveService = result;
        return redis_service_1.Redis.GetAllRunningRasaModelFromTest();
    })
        .then(function (result) {
        var promiseArray = [];
        var index = 0;
        if (result !== null) {
            var _loop_1 = function (key) {
                var value = JSON.parse(result[key]);
                var timeOut = index * 1000;
                if (moment.utc(value.created).add(45, 'minutes').isBefore(moment().utc())) {
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            return resolve(cognitiveService.StopApp(value.id, value.port, 'TEST'));
                        }, timeOut);
                    }));
                }
                index++;
            };
            for (var key in result) {
                _loop_1(key);
            }
        }
        return Promise.all(promiseArray);
    }).then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_STOP_EXPIRED_MODELS_FROM_TEST';
        logger.error('[schedule.task::StopAllRunningModelsFromTest] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
function InitBatchSchedule(executionCode) {
    var batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        var promiseArray = [];
        var startRange01 = moment();
        startRange01.set('hour', 0);
        startRange01.set('minute', 0);
        startRange01.set('second', 0);
        var endRange01 = moment();
        endRange01.set('hour', 0);
        endRange01.set('minute', 4);
        endRange01.set('second', 59);
        var startRange02 = moment();
        startRange02.set('hour', 12);
        startRange02.set('minute', 0);
        startRange02.set('second', 0);
        var endRange02 = moment();
        endRange02.set('hour', 12);
        endRange02.set('minute', 4);
        endRange02.set('second', 59);
        var firstRange = moment().isBetween(startRange01, endRange01);
        var secondRange = moment().isBetween(startRange02, endRange02);
        if (firstRange || secondRange) {
            logger.info('[schedule.task::InitBatchSchedule] Registering scheduleExecutions on redis at:', moment().format('YYYY-MM-DD HH:mm:ss'));
            promiseArray.push(batchScheduleImplementation.GetNewScheduledBatchExecutions(executionCode));
            if (firstRange && [1, 4].includes(moment().isoWeekday())) {
                logger.info('[schedule.task::InitBatchSchedule] Generating new scheduleExecutions on db at:', moment().format('YYYY-MM-DD HH:mm:ss'));
                promiseArray.push(batchScheduleImplementation.GenerateNewScheduleBatchExecution(executionCode));
            }
        }
        return Promise.all(promiseArray);
    })
        .then(function (_) {
        return Promise.resolve(true);
    })
        .catch(function (error) {
        var errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
        logger.error('[schedule.task::InitBatchSchedule] Error: ' + JSON.stringify(error));
        return Promise.resolve(false);
    });
}
