"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiUserImplementation = void 0;
var log4js = require("log4js");
var sy = require("systeminformation");
var promises_shared_1 = require("../shared/promises.shared");
var md5 = require("md5");
var mssql_service_1 = require("../db/mssql.service");
var platform_service_1 = require("../services/platform.service");
var logger = log4js.getLogger('Task');
var icModule = require('intermediate-component-sa');
var ApiUserImplementation = (function () {
    function ApiUserImplementation() {
    }
    ApiUserImplementation.prototype.GetSystemInformation = function (guid) {
        var systemInformationData = {
            installationId: guid,
            macAddress: '',
            publicIp: ''
        };
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[ApiUserImplementation::GetSystemInformation] guid:', guid);
            var sys = sy.networkInterfaces();
            return sys;
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_NETWORK_INTERFACE_INFORMATION';
            if (result && result.length) {
                logger.info('[ApiUserImplementation::GetSystemInformation] guid:', guid, 'getting result[0].mac:', result[0].mac);
                var myNetworkInterface = null;
                myNetworkInterface = result.filter(function (r) { return r.operstate === 'up'; })[0];
                systemInformationData.macAddress = myNetworkInterface.mac;
                systemInformationData.publicIp = myNetworkInterface.ip4;
            }
            logger.info('[ApiUserImplementation::GetSystemInformation] guid:', guid, 'getting systemInformationData:', systemInformationData);
            return Promise.resolve(systemInformationData);
        }).catch(function (err) {
            logger.error('[ApiUserImplementation::GetSystemInformation] guid:', guid, 'getting error:', err);
            return Promise.resolve(err);
        });
    };
    ApiUserImplementation.prototype.GenerateMD5Content = function (data) {
        var result = '';
        if (data) {
            result += data.installationId;
            result += data.macAddress;
            logger.info('[ApiUserImplementation::GenerateMD5Content] getting result:', result);
            result = md5(result);
        }
        return result;
    };
    ApiUserImplementation.prototype.GetApiUsers = function (vcc) {
        var _this = this;
        var systemInformationData = null;
        var installationId;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[ApiUserImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            var body = { vcc: vcc };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USERS_GET', body);
        }).then(function (result) {
            logger.info('[ApiUserImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        }).catch(function (err) {
            logger.error('[ApiUserImplementation::GetPlatformAvailableLicenses] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    ApiUserImplementation.prototype.SaveApiUser = function (vcc, userApi, pin, userSpeech) {
        var _this = this;
        var systemInformationData = null;
        var installationId;
        var pin_md5;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            if (!result.installationId) {
                throw 'INSTALLATIONID_MISSING';
            }
            installationId = result.installationId;
            pin_md5 = _this.GenerateUserPinMD5(vcc, userApi, pin, installationId);
            var body = {
                installationId: installationId,
                vcc: vcc,
                userApi: userApi,
                pin: pin_md5,
                userSpeech: userSpeech
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USER_SAVE', body);
        }).then(function (result) {
            logger.info('[ApiUserImplementation:SaveApiUser] Getting result:', result, 'from external server');
            if (result.status) {
                return Promise.resolve(pin_md5);
            }
            else {
                return Promise.resolve(null);
            }
        }).catch(function (err) {
            logger.error('[ApiUserImplementation::SaveApiUser] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    ApiUserImplementation.prototype.CreateApiUser = function (vcc, userApi, pin, userSpeech) {
        var _this = this;
        var systemInformationData = null;
        var installationId;
        var pin_md5;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            if (!result.installationId) {
                throw 'INSTALLATIONID_MISSING';
            }
            installationId = result.installationId;
            pin_md5 = _this.GenerateUserPinMD5(vcc, userApi, pin, installationId);
            var body = {
                installationId: installationId,
                vcc: vcc,
                userApi: userApi,
                pin: pin_md5,
                userSpeech: userSpeech
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USER_CREATE', body);
        }).then(function (result) {
            logger.info('[ApiUserImplementation:SaveApiUser] Getting result:', result, 'from external server');
            if (result.status) {
                return Promise.resolve(pin_md5);
            }
            else {
                return Promise.resolve(null);
            }
        }).catch(function (err) {
            logger.error('[ApiUserImplementation::SaveApiUser] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    ApiUserImplementation.prototype.DeleteApiUser = function (vcc, user) {
        var _this = this;
        var installationId;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[ApiUserImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            var body = {
                vcc: vcc,
                user: user
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USER_DELETE', body);
        }).then(function (result) {
            logger.info('[ApiUserImplementation:SaveApiUser] Getting result:', result, 'from external server');
            return Promise.resolve(result.status);
        }).catch(function (err) {
            logger.error('[ApiUserImplementation::SaveApiUser] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    ApiUserImplementation.prototype.DeleteApiUsers = function (vcc, user) {
        var _this = this;
        var installationId;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[ApiUserImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            var body = {
                vcc: vcc,
                users: user
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USERS_DELETE', body);
        }).then(function (result) {
            logger.info('[ApiUserImplementation:SaveApiUser] Getting result:', result, 'from external server');
            return Promise.resolve(result.status);
        }).catch(function (err) {
            logger.error('[ApiUserImplementation::SaveApiUser] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    ApiUserImplementation.prototype.GenerateUserPinMD5 = function (vcc, user, pin, installationId) {
        var result = null;
        result = user;
        result += '+';
        result += pin;
        result += '@';
        result += vcc;
        result += ':';
        result += installationId;
        logger.info('[ApiUserImplementation::GenerateUserPinMD5] Getting result:', result);
        result = md5(result);
        return result;
    };
    ApiUserImplementation.prototype.ValidateUserPin = function (vcc, user, pin) {
        var _this = this;
        var systemInformationData = null;
        var installationId;
        var apiUser = '';
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetApiUser(vcc, user);
        }).then(function (result) {
            if (result) {
                apiUser = result.user;
            }
            else {
                throw 'API_USER_NOT_FOUND';
            }
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[ApiUserImplementation::ValidateUserPin] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var userPinMD5 = _this.GenerateUserPinMD5(vcc, user, pin, systemInformationData.installationId);
            logger.info('[ApiUserImplementation::ValidateUserPin] Getting userPinMD5:', userPinMD5);
            var body = {
                platformHash: plaformHash,
                vcc: vcc,
                apiUser: apiUser,
                user: user,
                pin: userPinMD5,
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_API_USER_AUTH', body);
        }).then(function (result) {
            logger.info('[ApiUserImplementation::ValidateUserPin] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        }).catch(function (err) {
            logger.error('[ApiUserImplementation::ValidateUserPin] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    return ApiUserImplementation;
}());
exports.ApiUserImplementation = ApiUserImplementation;
