"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiarizationAnalysisImplementation = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var config_1 = require("../config/config");
var redis_service_1 = require("../db/redis.service");
var file_manager_1 = require("../shared/file-manager");
var promises_shared_1 = require("../shared/promises.shared");
var cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
var billing_implementation_1 = require("./billing.implementation");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'DiarizationAnalysisImplementation';
var DiarizationAnalysisImplementation = (function () {
    function DiarizationAnalysisImplementation() {
    }
    DiarizationAnalysisImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RATIO_DIARIZATION_ANALYSIS;
        var executeResult = true;
        return promises_shared_1.SleepPromise(config_1.SLEEP_DIARIZATION_ANALYSIS)
            .then(function (result) {
            return redis_service_1.Redis.LRange('DiarizationAnalysis', 0, ratio - 1);
        }).then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(_this.DiarizationConvertExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('DiarizationAnalysis', 0, ratio - 1);
        }).then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        }).then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[DiarizationAnalysisImplementation::Execute] executionCode:', executionCode, 'getting error results:', errorMessage);
            return Promise.resolve(false);
        });
    };
    DiarizationAnalysisImplementation.prototype.DiarizationConvertExecute = function (executionCode) {
        var _this = this;
        var currentBlockPromise = 'BLOCK_BEFORE_PROMISE';
        var storage = '';
        var currentQueueId = '';
        var processInteraction;
        var isCancel = false;
        var vcc = '';
        var processId = '';
        var interactionType = '';
        var interactionId = '';
        var interactionSection = 0;
        var interactionPart = 0;
        var batchTypeName = '';
        var batchTypeId = '';
        return redis_service_1.Redis.RPopLPush('DiarizationAnalysis', 'DiarizationAnalysisExecute')
            .then(function (queueId) {
            currentBlockPromise = 'BLOCK_TO_GET_PROCESS_INTERACTION';
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                var item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then(function (processInteractionResult) {
            currentBlockPromise = 'BLOCK_TO_LAUNCH_MAIN_FUNCTION';
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                if (processInteraction.statusDescription === 'pendingIdentifyAgent') {
                    return _this.IdentifyAgent(processInteraction, currentQueueId, executionCode);
                }
                else if (processInteraction.statusDescription === 'pendingInvertChannels' || processInteraction.channelchange) {
                    return _this.InvertChannels(processInteraction, currentQueueId, executionCode);
                }
                return null;
            }
            else {
                if (currentQueueId)
                    logger.warn('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return null;
            }
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_UPDATE_INTERACTION';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return null;
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_LAUNCH_FINISHING_FUNCTION';
            var promiseArray = [];
            var billing = new billing_implementation_1.BillingImplementation();
            if (result
                && processInteraction
                && processInteraction.status === 4
                && processInteraction.statusDescription === 'pendingSentimentAnalysis') {
                logger.info('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                promiseArray.push(redis_service_1.Redis.CreateQueueL('SentimentAnalysis', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationAnalysisExecute', currentQueueId));
            }
            else if (result
                && processInteraction
                && (processInteraction.status === 3
                    ||
                        (processInteraction.status === 4 && processInteraction.statusDescription !== 'pendingSentimentAnalysis'))
                && processInteraction.retry < config_1.RETRY_DIARIZATION_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('DiarizationAnalysis', currentQueueId, config_1.RATE_RECURSIVE_DIARIZATION_ANALYSIS * config_1.RATIO_DIARIZATION_ANALYSIS * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationAnalysisExecute', currentQueueId));
            }
            else if (result
                && processInteraction
                && processInteraction.status === -4
                && processInteraction.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationConvertExecute', currentQueueId));
            }
            else if (processInteraction && processInteraction.retry >= config_1.RETRY_DIARIZATION_ANALYSIS) {
                logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationAnalysisExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                var interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, batchTypeName, batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('DiarizationAnalysisExecute', currentQueueId));
            }
            else {
                logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.resolve(true);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_UPDATE_REDIS';
            var promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_DIARIZATION_ANALYSIS) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, batchTypeId, batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            currentBlockPromise = 'BLOCK_TO_FINISH_EXECUTION';
            return Promise.resolve(true);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationAnalysisImplementation::DiarizationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    DiarizationAnalysisImplementation.prototype.IdentifyAgent = function (interaction, currentQueueId, executionCode) {
        var _this = this;
        var path = '';
        var jsonContent = null;
        var cognitiveService;
        var cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
        var phrases = null;
        var agentResults = [];
        var clientResults = [];
        var agentEvaluationResult = [];
        var clientEvaluationResult = [];
        var needInvertChannels = false;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            var regex = /\.(mp3|wav)$/;
            var jsonFileName = interaction.fileName.replace(regex, '.json');
            path = config_1.TRANSCRIPTION_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return file_manager_1.ReadFileContent(path, 'utf-8');
        }).then(function (result) {
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
                phrases = _this.GeneratePhrases(jsonContent.results, config_1.COGNITIVE_SERVICES_RASA_CHARS_PER_PHRASE);
            }
            catch (ex) {
                jsonContent = '';
                throw 'SERVER_ERROR_GET_JSON_FAILED';
            }
            return Promise.resolve(phrases);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_PHRASES_NOT_FOUND';
            return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_COGNITIVE_SERVICE';
            cognitiveService = result;
            var promiseArray = [];
            phrases.agent.map(function (ap, i) {
                var appId = interaction.vcc + '::diarization-' + interaction.language;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        return resolve(cognitiveAnalysis.ModelQuery(cognitiveService, null, null, appId, ap, appId, executionCode, currentQueueId));
                    }, i * 1000);
                }));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_AGENT_EVALUATION_FAILED';
            logger.debug('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting agent evaluation result:', result);
            agentEvaluationResult = result;
            var promiseArray = [];
            phrases.client.map(function (ap, i) {
                var appId = interaction.vcc + '::diarization-' + interaction.language;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(function () {
                        return resolve(cognitiveAnalysis.ModelQuery(cognitiveService, null, null, appId, ap, appId, executionCode, currentQueueId));
                    }, i * 1000);
                }));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_CLIENT_EVALUATION_FAILED';
            logger.debug('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting client evaluation result:', result);
            clientEvaluationResult = result;
            var maxAgentScore = [];
            var maxClientScore = [];
            var agentMaxScore = Math.max.apply(Math, agentEvaluationResult.map(function (aer, i) {
                agentResults.push({
                    phrase: phrases.agent[i],
                    scores: aer.intents
                });
                var agentItem = aer.intents.find(function (i) { return i.intent === 'Agent'; });
                if (agentItem) {
                    maxAgentScore.push({
                        phrase: phrases.agent[i],
                        score: agentItem.score
                    });
                    return agentItem.score;
                }
                else
                    return 0;
            }));
            var clientMaxScore = Math.max.apply(Math, clientEvaluationResult.map(function (cer, i) {
                clientResults.push({
                    phrase: phrases.client[i],
                    scores: cer.intents
                });
                var clientItem = cer.intents.find(function (i) { return i.intent === 'Agent'; });
                if (clientItem) {
                    maxClientScore.push({
                        phrase: phrases.client[i],
                        score: clientItem.score
                    });
                    return clientItem.score;
                }
                else
                    return 0;
            }));
            logger.debug('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting max client evaluation result:', maxClientScore.filter(function (mcs) { return mcs.score === clientMaxScore; }));
            logger.debug('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting max agent evaluation result:', maxAgentScore.filter(function (mas) { return mas.score === agentMaxScore; }));
            if (agentMaxScore > clientMaxScore) {
                needInvertChannels = false;
            }
            else if (agentMaxScore === clientMaxScore) {
                var promAgent = agentEvaluationResult.reduce(function (sum, current) {
                    var agentItem = current.intents.find(function (i) { return i.intent === 'Agent'; });
                    return sum + agentItem ? agentItem.score : 0;
                }, 0) / agentEvaluationResult.length;
                var promclient = clientEvaluationResult.reduce(function (sum, current) {
                    var clientItem = current.intents.find(function (i) { return i.intent === 'Agent'; });
                    return sum + clientItem ? clientItem.score : 0;
                }, 0) / clientEvaluationResult.length;
                if (promAgent > promclient)
                    needInvertChannels = false;
                else if (promAgent < promclient)
                    needInvertChannels = true;
            }
            else {
                needInvertChannels = true;
            }
            var resultsObject = {
                Model: interaction.vcc + '::diarization-' + interaction.language,
                AllPhrases: phrases.allPhrases,
                AgentPhrases: agentResults,
                ClientPhrases: clientResults
            };
            return _this.SaveDiarizationResults(interaction, resultsObject, currentQueueId, executionCode);
        })
            .then(function (result) {
            return Promise.resolve('OK');
        })
            .then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 4;
                interaction['statusDescription'] = needInvertChannels ? 'pendingInvertChannels' : 'pendingSentimentAnalysis';
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -4;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'IdentifyAgent';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationAnalysisImplementation::IdentifyAgent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -4;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'IdentifyAgent';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    };
    DiarizationAnalysisImplementation.prototype.InvertChannels = function (interaction, currentQueueId, executionCode) {
        var path = '';
        var jsonContent = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            var regex = /\.(mp3|wav)$/;
            var jsonFileName = interaction.fileName.replace(regex, '.json');
            path = config_1.TRANSCRIPTION_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return file_manager_1.ReadFileContent(path, 'utf-8');
        }).then(function (result) {
            jsonContent = result;
            var agentRegex = new RegExp("(?<=\"channel\"\\:)\"Agent\"", 'gm');
            var clientRegex = new RegExp("(?<=\"channel\"\\:)\"Caller\"", 'gm');
            jsonContent = jsonContent.replace(agentRegex, '"OldAgent"');
            jsonContent = jsonContent.replace(clientRegex, '"OldCaller"');
            var oldAgentRegex = new RegExp("(?<=\"channel\"\\:)\"OldAgent\"", 'gm');
            var oldClientRegex = new RegExp("(?<=\"channel\"\\:)\"OldCaller\"", 'gm');
            jsonContent = jsonContent.replace(oldAgentRegex, '"Caller"');
            jsonContent = jsonContent.replace(oldClientRegex, '"Agent"');
            return Promise.resolve(true);
        }).then(function (result) {
            return file_manager_1.WriteFileContent(path, jsonContent, 'utf-8', true);
        }).then(function (result) {
            return Promise.resolve('OK');
        }).then(function (result) {
            if (result === 'OK') {
                interaction['status'] = 4;
                interaction['statusDescription'] = 'pendingSentimentAnalysis';
                interaction['channelsChanged'] = true;
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
            }
            else {
                logger.error('[DiarizationAnalysisImplementation::InvertChannels] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -4;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'InvertChannels';
                interaction['errorMessage'] = result;
                interaction['errorDetail'] = result;
            }
            return Promise.resolve(interaction);
        }).catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[DiarizationAnalysisImplementation::InvertChannels] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -4;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'InvertChannels';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    };
    DiarizationAnalysisImplementation.prototype.SaveDiarizationResults = function (interaction, jsonContent, currentQueueId, executionCode) {
        var diarizationFolder = config_1.DIARIZATION_FOLDER + '/' + interaction.vcc;
        var regex = /\.(mp3|wav|json)$/;
        var jsonFileName = interaction.fileName.replace(regex, '.json');
        var jsonFilePath = null;
        jsonFilePath = diarizationFolder;
        jsonFilePath += '/';
        jsonFilePath += jsonFileName;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return file_manager_1.CreateFolder(diarizationFolder);
        })
            .then(function (result) {
            logger.info('[DiarizationAnalysisImplementation::SaveDiarizationResults] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'saving results on:', jsonFilePath);
            return file_manager_1.WriteFileContent(jsonFilePath, JSON.stringify(jsonContent), 'utf-8');
        })
            .then(function (result) {
            return Promise.resolve(true);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[DiarizationAnalysisImplementation::SaveDiarizationResults] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.reject(errorMessage);
        });
    };
    DiarizationAnalysisImplementation.prototype.GeneratePhrases = function (jsonWords, charsPerPhrase) {
        var allPhrases = [];
        var agentPhrases = [];
        var clientPhrases = [];
        var currentEntity = '';
        var currentPhrase = '';
        var currentPhraseStartTime = 0;
        var currentPhraseEndTime = 0;
        jsonWords.sort(function (a, b) { return (a.start_time > b.start_time) ? 1 : -1; });
        jsonWords.map(function (jw, i) {
            var currentContent = '';
            currentPhraseEndTime = jw.end_time;
            if (currentEntity === jw.channel) {
                currentContent += jw.type === 'word' ? ' ' : '';
            }
            else {
                if (currentEntity !== '') {
                    allPhrases.push({
                        entity: currentEntity,
                        phrase: currentPhrase,
                        startTime: currentPhraseStartTime,
                        endTime: currentPhraseEndTime
                    });
                    if (currentPhrase.length) {
                        if (currentEntity === 'Agent')
                            agentPhrases.push(currentPhrase);
                        if (currentEntity === 'Caller')
                            clientPhrases.push(currentPhrase);
                    }
                }
                currentEntity = jw.channel;
                currentPhrase = '';
                currentPhraseStartTime = jw.start_time;
            }
            currentContent += jw.alternatives[0].content;
            if ((currentPhrase.length + currentContent.length) > charsPerPhrase
                ||
                    (i + 1 === jsonWords.length)) {
                allPhrases.push({
                    entity: currentEntity,
                    phrase: currentPhrase,
                    startTime: currentPhraseStartTime,
                    endTime: currentPhraseEndTime
                });
                if (currentPhrase.length) {
                    if (currentEntity === 'Agent')
                        agentPhrases.push(currentPhrase);
                    if (currentEntity === 'Caller')
                        clientPhrases.push(currentPhrase);
                }
                currentPhrase = '';
            }
            else {
                currentPhrase += currentContent;
            }
        });
        return {
            agent: agentPhrases,
            client: clientPhrases,
            allPhrases: allPhrases
        };
    };
    return DiarizationAnalysisImplementation;
}());
exports.DiarizationAnalysisImplementation = DiarizationAnalysisImplementation;
